@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # This file is part of python-opsi.
-# Copyright (C) 2006-2014 uib GmbH <info@uib.de>
+# Copyright (C) 2006-2015 uib GmbH <info@uib.de>
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -30,8 +30,6 @@ need to set up you backends, ACL, multiplexing etc. yourself.
 :license: GNU Affero General Public License version 3
 """
 
-__version__ = '4.0.6.1'
-
 import inspect
 import new
 import os
@@ -40,6 +38,22 @@ import socket
 import sys
 import types
 
+from OPSI.Backend.Backend import (Backend, BackendModificationListener,
+	ConfigDataBackend, ExtendedBackend, ExtendedConfigDataBackend,
+	ModificationTrackingBackend,
+	getArgAndCallString)
+from OPSI.Backend.Depotserver import DepotserverBackend
+from OPSI.Backend.HostControl import HostControlBackend
+from OPSI.Backend.HostControlSafe import HostControlSafeBackend
+from OPSI.Backend.JSONRPC import JSONRPCBackend
+from OPSI.Logger import Logger, LOG_INFO
+from OPSI.Object import BaseObject, mandatoryConstructorArgs
+from OPSI.Object import *  # this is needed for dynamic extension loading
+from OPSI.Types import *  # this is needed for dynamic extension loading
+from OPSI.Util import objectToBeautifiedText, getfqdn
+from OPSI.Util.File.Opsi import BackendACLFile, BackendDispatchConfigFile, OpsiConfFile
+from OPSI.Util.MessageBus import MessageBusClient
+
 if os.name == 'posix':
 	import grp
 	import PAM
@@ -48,17 +62,7 @@ elif os.name == 'nt':
 	import win32net
 	import win32security
 
-from OPSI.Logger import Logger, LOG_INFO
-from OPSI.Types import *
-from OPSI.Object import BaseObject
-from OPSI.Backend.Backend import *
-from OPSI.Backend.JSONRPC import JSONRPCBackend
-from OPSI.Backend.Depotserver import DepotserverBackend
-from OPSI.Backend.HostControl import HostControlBackend
-from OPSI.Backend.HostControlSafe import HostControlSafeBackend
-from OPSI.Util import objectToBeautifiedText, getfqdn
-from OPSI.Util.File.Opsi import BackendACLFile, BackendDispatchConfigFile, OpsiConfFile
-from OPSI.Util.MessageBus import MessageBusClient
+__version__ = '4.0.6.39'
 
 logger = Logger()
 
@@ -70,7 +74,7 @@ except ImportError:
 	DISTRIBUTOR = 'unknown'
 
 try:
-	f = os.popen('lsb_release -d 2>/dev/null')
+	f = os.popen('lsb_release -d 2>&1 /dev/null')
 	DISTRIBUTION = f.read().split(':')[1].strip()
 	f.close()
 except Exception as error:
@@ -78,7 +82,7 @@ except Exception as error:
 	DISTRIBUTION = 'unknown'
 
 try:
-	f = os.popen('lsb_release -r 2>/dev/null')
+	f = os.popen('lsb_release -r 2>&1 /dev/null')
 	DISTRELEASE = f.read().split(':')[1].strip()
 	f.close()
 except Exception as error:
@@ -135,7 +139,48 @@ class MessageBusNotifier(BackendModificationListener):
 
 
 class BackendManager(ExtendedBackend):
+	"""
+	The BackendManager manages the backend and glues together various parts.
+
+	This includes extending the backends, dispatching calls to backends,
+	limiting the access	through ACL.
+	"""
+
 	def __init__(self, **kwargs):
+		"""
+		Creating a BackendManager.
+
+		If no configuration is given a default config for accessing the
+		local backend as configured through the files in
+		/etc/opsi/backendManager/ will be used.
+
+		The constructor takes several parameters and these are all optional.
+
+		:param username: A username for authentication.
+		:type username: str
+		:param password: The corresponding password.
+		:type password: str
+		:param backend: A backend to use.
+		:param dispatchconfig: A pre-definded dispatch config to use.
+		:param dispatchconfigfile: The configuration file for dispatching.
+		:type dispatchconfigfile: str
+		:param backendconfigdir: The location of backend configurations.
+		:type backendconfigdir: str
+		:param depotbackend: Allow depot actions?
+		:type depotbackend: bool
+		:param hostcontrolbackend: Allow controlling hosts?
+		:type hostcontrolbackend: bool
+		:param hostcontrolsafebackend: Allow controlling hosts (safe variant)?
+		:type hostcontrolsafebackend: bool
+		:param extensionconfigdir: The directory where backend extensions can be found.
+		:type extensionconfigdir: str
+		:param extend: Extend the backends?
+		:type extend: bool
+		:param acl: An access control list (ACL) configuration to use.
+		:type acl: [[str, ]]
+		:param aclfile: Load the ACL from this file.
+		:type aclfile: str
+		"""
 		self._backend = None
 		self._backendConfigDir = None
 		self._options = {}
@@ -159,6 +204,17 @@ class BackendManager(ExtendedBackend):
 		startReactor = True
 		loadBackend = None
 
+		if not kwargs:
+			kwargs = {
+				"dispatchConfigFile": u'/etc/opsi/backendManager/dispatch.conf',
+				"backendConfigDir": u'/etc/opsi/backends',
+				"extensionConfigDir": u'/etc/opsi/backendManager/extend.d',
+				"depotBackend": True,
+				"hostControlBackend": True,
+				"hostControlSafeBackend": True,
+			}
+			logger.debug("No config given, using {0!r}".format(kwargs))
+
 		for (option, value) in kwargs.items():
 			option = option.lower()
 			if option == 'username':
@@ -166,7 +222,7 @@ class BackendManager(ExtendedBackend):
 			elif option == 'password':
 				password = value
 			elif option == 'backend':
-				if type(value) in (str, unicode):
+				if isinstance(value, (str, unicode)):
 					loadBackend = value
 				else:
 					self._backend = value
@@ -206,7 +262,7 @@ class BackendManager(ExtendedBackend):
 
 		if dispatch:
 			logger.info(u"* BackendManager is creating BackendDispatcher")
-			self._backend = BackendDispatcher(context = self, **kwargs)
+			self._backend = BackendDispatcher(context=self, **kwargs)
 			# self._backend is now a BackendDispatcher which is a ConfigDataBackend
 
 		if messageBusNotifier:
@@ -220,9 +276,11 @@ class BackendManager(ExtendedBackend):
 			# DepotserverBackend/BackendExtender need ExtendedConfigDataBackend backend
 			self._backend = ExtendedConfigDataBackend(self._backend)
 			# self._backend is now an ExtendedConfigDataBackend
+
 		if depotBackend:
 			logger.info(u"* BackendManager is creating DepotserverBackend")
 			self._backend = DepotserverBackend(self._backend)
+
 		if hostControlBackend:
 			logger.info(u"* BackendManager is creating HostControlBackend")
 			hcc = {}
@@ -231,6 +289,7 @@ class BackendManager(ExtendedBackend):
 			except Exception as e:
 				logger.error(e)
 			self._backend = HostControlBackend(self._backend, **hcc)
+
 		if hostControlSafeBackend:
 			logger.info(u"* BackendManager is creating HostControlBackend")
 			hcc = {}
@@ -239,9 +298,11 @@ class BackendManager(ExtendedBackend):
 			except Exception as e:
 				logger.error(e)
 			self._backend = HostControlSafeBackend(self._backend, **hcc)
+
 		if accessControl:
 			logger.info(u"* BackendManager is creating BackendAccessControl")
-			self._backend = BackendAccessControl(backend = self._backend, **kwargs)
+			self._backend = BackendAccessControl(backend=self._backend, **kwargs)
+
 		if extensionConfigDir or extensionClass:
 			logger.info(u"* BackendManager is creating BackendExtender")
 			self._backend = BackendExtender(self._backend, **kwargs)
@@ -269,7 +330,7 @@ class BackendManager(ExtendedBackend):
 		backendConfigFile = os.path.join(self._backendConfigDir, '%s.conf' % name)
 		if not config['module']:
 			raise BackendConfigurationError(u"No module defined in backend config file '%s'" % backendConfigFile)
-		if not type(config['config']) is dict:
+		if not isinstance(config['config'], dict):
 			raise BackendConfigurationError(u"Bad type for config var in backend config file '%s', has to be dict" % backendConfigFile)
 		config['config']['name'] = name
 		exec(u'from %s import %sBackend' % (config['module'], config['module']))
@@ -286,7 +347,7 @@ class BackendDispatcher(Backend):
 		Backend.__init__(self, **kwargs)
 
 		self._dispatchConfigFile = None
-		self._dispatchConfig = None
+		self._dispatchConfig = []
 		self._dispatchIgnoreModules = []
 		self._backendConfigDir = None
 		self._backends = {}
@@ -309,16 +370,39 @@ class BackendDispatcher(Backend):
 		if self._dispatchConfigFile:
 			logger.info(u"Loading dispatch config file '%s'" % self._dispatchConfigFile)
 			self.__loadDispatchConfig()
+
 		if not self._dispatchConfig:
 			raise BackendConfigurationError(u"Dispatcher not configured")
+
 		self.__loadBackends()
 		self._createInstanceMethods()
 
+	def __repr__(self):
+		additionalInformation = []
+		try:
+			if self._dispatchIgnoreModules:
+				additionalInformation.append('dispatchIgnoreModules={0!r}'.format(self._dispatchIgnoreModules))
+
+			if self._dispatchConfigFile:
+				additionalInformation.append('dispatchConfigFile={0!r}'.format(self._dispatchConfigFile))
+			elif self._dispatchConfig:
+				additionalInformation.append('dispatchConfig={0!r}'.format(self._dispatchConfig))
+
+			if self._context != self:
+				additionalInformation.append('context={0!r}'.format(self._context))
+		except AttributeError:
+			# Can happen during initialisation
+			pass
+
+		return '<{0}({1})>'.format(self.__class__.__name__, ', '.join(additionalInformation))
+
 	def __loadDispatchConfig(self):
 		if not self._dispatchConfigFile:
 			raise BackendConfigurationError(u"No dispatch config file defined")
+
 		if not os.path.exists(self._dispatchConfigFile):
 			raise BackendConfigurationError(u"Dispatch config file '%s' not found" % self._dispatchConfigFile)
+
 		try:
 			self._dispatchConfig = BackendDispatchConfigFile(self._dispatchConfigFile).parse()
 			logger.debug(u"Read dispatch config from file '%s':" % self._dispatchConfigFile)
@@ -330,16 +414,14 @@ class BackendDispatcher(Backend):
 		backends = set()
 		if not self._backendConfigDir:
 			raise BackendConfigurationError(u"Backend config dir not given")
+
 		if not os.path.exists(self._backendConfigDir):
 			raise BackendConfigurationError(u"Backend config dir '%s' not found" % self._backendConfigDir)
 
-		for i in xrange(len(self._dispatchConfig)):
-			if not type(self._dispatchConfig[i][1]) is list:
-				self._dispatchConfig[i][1] = [self._dispatchConfig[i][1]]
-
-			for value in self._dispatchConfig[i][1]:
+		for regex, backend in self._dispatchConfig:
+			for value in forceList(backend):
 				if not value:
-					raise BackendConfigurationError(u"Bad dispatcher config '%s'" % self._dispatchConfig[i])
+					raise BackendConfigurationError(u"Bad dispatcher config: {0!r} has empty target backend: {1!r}".format(regex, backend))
 
 				backends.add(value)
 
@@ -357,7 +439,7 @@ class BackendDispatcher(Backend):
 				logger.notice(u"Ignoring module '%s', backend '%s'" % (l['module'], backend))
 				del self._backends[backend]
 				continue
-			if not type(l['config']) is dict:
+			if not isinstance(l['config'], dict):
 				raise BackendConfigurationError(u"Bad type for config var in backend config file '%s', has to be dict" % backendConfigFile)
 			backendInstance = None
 			l["config"]["context"] = self
@@ -366,64 +448,59 @@ class BackendDispatcher(Backend):
 
 	def _createInstanceMethods(self):
 		logger.debug(u"BackendDispatcher is creating instance methods")
-		for Class in (ConfigDataBackend,):#, ExtendedConfigDataBackend):
-			for member in inspect.getmembers(Class, inspect.ismethod):
-				methodName = member[0]
+		for Class in (ConfigDataBackend, ):  #  Also apply to ExtendedConfigDataBackend?
+			for methodName, functionRef in inspect.getmembers(Class, inspect.ismethod):
 				if methodName.startswith('_'):
 					# Not a public method
 					continue
 				logger.debug2(u"Found public %s method '%s'" % (Class.__name__, methodName))
 
 				if hasattr(self, methodName):
-					logger.debug(u"%s: overwriting method %s" % (self.__class__.__name__, methodName))
+					logger.debug(u"{0}: skipping already present method {1}".format(self.__class__.__name__, methodName))
 					continue
 
 				methodBackends = []
-				for i in xrange(len(self._dispatchConfig)):
-					(regex, backends) = self._dispatchConfig[i]
+				for regex, backends in self._dispatchConfig:
 					if not re.search(regex, methodName):
 						continue
 
 					for backend in forceList(backends):
 						if backend not in self._backends:
-							logger.debug(u"Ignoring backend '%s': backend not available" % backend)
+							logger.debug(u"Ignoring backend {0!r}: backend not available".format(backend))
 							continue
 						methodBackends.append(backend)
-					logger.debug(u"'%s' matches method '%s', dispatching to backends: %s" % (regex, methodName, u', '.join(methodBackends)))
+					logger.debug(u"{0!r} matches method {1!r}, dispatching to backends: {2}".format(regex, methodName, u', '.join(methodBackends)))
 					break
 
 				if not methodBackends:
 					continue
 
-				(argString, callString) = getArgAndCallString(member[1])
+				argString, callString = getArgAndCallString(functionRef)
 
 				exec(u'def %s(self, %s): return self._dispatchMethod(%s, "%s", %s)' % (methodName, argString, methodBackends, methodName, callString))
 				setattr(self, methodName, new.instancemethod(eval(methodName), self, self.__class__))
 
 	def _dispatchMethod(self, methodBackends, methodName, **kwargs):
-		logger.debug(u"Dispatching method '%s' to backends: %s" % (methodName, methodBackends))
+		logger.debug(u"Dispatching method {0!r} to backends: {1}".format(methodName, methodBackends))
 		result = None
-		objectIdents = []
+
 		for methodBackend in methodBackends:
 			meth = getattr(self._backends[methodBackend]["instance"], methodName)
-			res =  meth(**kwargs)
-			if type(res) is types.ListType:
-				# Remove duplicates
-				newRes = []
-				for r in res:
-					if isinstance(r, BaseObject):
-						ident = r.getIdent()
-						if ident in objectIdents:
-							continue
-						objectIdents.append(ident)
-					newRes.append(r)
-				res = newRes
-			if type(result) is types.ListType and type(res) is types.ListType:
+			res = meth(**kwargs)
+
+			# TODO: handling of generators?
+			if isinstance(result, list) and isinstance(res, list):
 				result.extend(res)
-			elif type(result) is types.DictType and type(res) is types.DictType:
+			elif isinstance(result, dict) and isinstance(res, dict):
 				result.update(res)
+			elif isinstance(result, set) and isinstance(res, set):
+				result = result.union(res)
+			elif isinstance(result, tuple) and isinstance(res, tuple):
+				result = result + res
 			elif res is not None:
 				result = res
+
+		logger.debug2(u"Finished dispatching method {0!r}".format(methodName))
 		return result
 
 	def backend_setOptions(self, options):
@@ -454,7 +531,7 @@ class BackendExtender(ExtendedBackend):
 			if not isinstance(backend, BackendAccessControl) or (not isinstance(backend._backend, ExtendedBackend) and not isinstance(backend._backend, BackendDispatcher)):
 				raise Exception("BackendExtender needs instance of ExtendedBackend or BackendDispatcher as backend, got %s" % backend.__class__.__name__)
 
-		ExtendedBackend.__init__(self, backend, overwrite = kwargs.get('overwrite', True))
+		ExtendedBackend.__init__(self, backend, overwrite=kwargs.get('overwrite', True))
 
 		self._extensionConfigDir = None
 		self._extensionClass = None
@@ -470,18 +547,17 @@ class BackendExtender(ExtendedBackend):
 
 	def __createExtensions(self):
 		if self._extensionClass:
-			for member in inspect.getmembers(self._extensionClass, inspect.ismethod):
-				methodName = member[0]
+			for methodName, functionRef in inspect.getmembers(self._extensionClass, inspect.ismethod):
 				if methodName.startswith('_'):
 					continue
-				logger.debug2(u"Extending %s with instancemethod: '%s'" % (self._backend.__class__.__name__, methodName))
-				new_function = new.function( member[1].func_code, member[1].func_globals, member[1].func_code.co_name )
-				new_method = new.instancemethod( new_function, self, self.__class__ )
-				setattr( self, methodName, new_method )
+				logger.debug2(u"Extending {0} with instancemethod: {1!r}".format(self._backend.__class__.__name__, methodName))
+				new_function = new.function(functionRef.func_code, functionRef.func_globals, functionRef.func_code.co_name)
+				new_method = new.instancemethod(new_function, self, self.__class__)
+				setattr(self, methodName, new_method)
 
 		if self._extensionConfigDir:
 			if not os.path.exists(self._extensionConfigDir):
-				logger.error(u"No extensions loaded: '%s' does not exist" % self._extensionConfigDir)
+				logger.error(u"No extensions loaded: extension directory {0!r} does not exist".format(self._extensionConfigDir))
 				return
 
 			try:
@@ -494,17 +570,16 @@ class BackendExtender(ExtendedBackend):
 					try:
 						logger.info(u"Reading config file '%s'" % confFile)
 						execfile(confFile)
+					except Exception as execError:
+						logger.logException(execError)
+						raise Exception(u"Error reading file {0!r}: {1}".format(confFile, execError))
 
-					except Exception as e:
-						logger.logException(e)
-						raise Exception(u"Error reading file '%s': %s" % (confFile, e))
-
-					for (key, val) in locals().items():
-						if type(val) == types.FunctionType:
+					for key, val in locals().items():
+						if isinstance(val, types.FunctionType):   # TODO: find a better way
 							logger.debug2(u"Extending %s with instancemethod: '%s'" % (self._backend.__class__.__name__, key))
 							setattr(self, key, new.instancemethod(val, self, self.__class__))
-			except Exception as e:
-				raise BackendConfigurationError(u"Failed to read extensions from '%s': %s" % (self._extensionConfigDir, e))
+			except Exception as error:
+				raise BackendConfigurationError(u"Failed to read extensions from '%s': %s" % (self._extensionConfigDir, error))
 
 
 class BackendAccessControl(object):
@@ -518,12 +593,15 @@ class BackendAccessControl(object):
 		self._acl = None
 		self._aclFile = None
 		self._pamService = 'common-auth'
-		self._userGroups = []
+		self._userGroups = set()
 		self._forceGroups = None
 		self._host = None
 		self._authenticated = False
 
-		if 'suse' in DISTRIBUTOR.lower():
+		if os.path.exists("/etc/pam.d/opsi-auth"):
+			# Prefering our own - if present.
+			self._pamService = 'opsi-auth'
+		elif 'suse' in DISTRIBUTOR.lower():
 			self._pamService = 'sshd'
 		elif 'centos' in DISTRIBUTOR.lower() or 'scientific' in DISTRIBUTOR.lower():
 			self._pamService = 'system-auth'
@@ -532,10 +610,6 @@ class BackendAccessControl(object):
 			if DISTRELEASE.startswith('6.'):
 				self._pamService = 'password-auth'
 
-		if os.path.exists("/etc/pam.d/opsi-auth"):
-			# Prefering our own - if present.
-			self._pamService = 'opsi-auth'
-
 		for (option, value) in kwargs.items():
 			option = option.lower()
 			if option == 'username':
@@ -574,14 +648,13 @@ class BackendAccessControl(object):
 			if re.search('^[^\.]+\.[^\.]+\.\S+$', self._username):
 				# Username starts with something like hostname.domain.tld:
 				# Assuming it is a host passing his FQDN as username
-				self._username = self._username.lower()
-
 				logger.debug(u"Trying to authenticate by opsiHostKey...")
+				self._username = self._username.lower()
 
 				if not hasattr(self._context, 'host_getObjects'):
 					raise Exception(u"Passed backend has no method 'host_getObjects', cannot authenticate host '%s'" % self._username)
 
-				host = self._context.host_getObjects(id = self._username)
+				host = self._context.host_getObjects(id=self._username)
 				if not host:
 					raise Exception(u"Host '%s' not found in backend %s" % (self._username, self._context))
 				self._host = host[0]
@@ -589,12 +662,10 @@ class BackendAccessControl(object):
 				if not self._host.opsiHostKey:
 					raise Exception(u"OpsiHostKey not found for host '%s'" % self._username)
 
-				logger.confidential(u"Client '%s', key sent '%s', key stored '%s'" \
-						% (self._username, self._password, self._host.opsiHostKey))
+				logger.confidential(u"Client '%s', key sent '%s', key stored '%s'" % (self._username, self._password, self._host.opsiHostKey))
 
-				if (self._password != self._host.opsiHostKey):
-					raise BackendAuthenticationError(u"OpsiHostKey authentication failed for host '%s': wrong key" \
-										% self._host.id)
+				if self._password != self._host.opsiHostKey:
+					raise BackendAuthenticationError(u"OpsiHostKey authentication failed for host '%s': wrong key" % self._host.id)
 
 				logger.info(u"OpsiHostKey authentication successful for host '%s'" % self._host.id)
 			else:
@@ -602,8 +673,7 @@ class BackendAccessControl(object):
 				logger.debug(u"Trying to authenticate by operating system...")
 				self._authenticateUser()
 				# Authentication did not throw exception => authentication successful
-				logger.info(u"Operating system authentication successful for user '%s', groups '%s'" \
-									% (self._username, ','.join(self._userGroups)))
+				logger.info(u"Operating system authentication successful for user '%s', groups '%s'" % (self._username, ','.join(self._userGroups)))
 		except Exception as e:
 			raise BackendAuthenticationError(u"%s" % e)
 
@@ -640,17 +710,16 @@ class BackendAccessControl(object):
 	def _createInstanceMethods(self):
 		protectedMethods = set()
 		for Class in (ExtendedConfigDataBackend, ConfigDataBackend, DepotserverBackend, HostControlBackend, HostControlSafeBackend):
-			methodnames = (method[0] for method in inspect.getmembers(Class, inspect.ismethod))
-			[protectedMethods.add(methodName) for methodName in
-			(name for name in methodnames if not name.startswith('_'))]
+			methodnames = (name for name, _ in inspect.getmembers(Class, inspect.ismethod) if not name.startswith('_'))
+			for methodName in methodnames:
+				protectedMethods.add(methodName)
 
-		for member in inspect.getmembers(self._backend, inspect.ismethod):
-			methodName = member[0]
+		for methodName, functionRef in inspect.getmembers(self._backend, inspect.ismethod):
 			if methodName.startswith('_'):
 				# Not a public method
 				continue
 
-			(argString, callString) = getArgAndCallString(member[1])
+			argString, callString = getArgAndCallString(functionRef)
 
 			if methodName in protectedMethods:
 				logger.debug2(u"Protecting %s method '%s'" % (Class.__name__, methodName))
@@ -664,28 +733,28 @@ class BackendAccessControl(object):
 	def _authenticateUser(self):
 		'''
 		Authenticate a user by the underlying operating system.
-		Throws BackendAuthenticationError on failure.
+
+		:raises BackendAuthenticationError: If authentication fails.
 		'''
-		if (os.name == 'posix'):
-			# Posix os => authenticate by PAM
+		if os.name == 'posix':
 			self._pamAuthenticateUser()
-		elif (os.name == 'nt'):
-			# NT os => authenticate by windows-login
+		elif os.name == 'nt':
 			self._winAuthenticateUser()
 		else:
-			# Other os, not implemented yet
 			raise NotImplementedError("Sorry, operating system '%s' not supported yet!" % os.name)
 
 	def _winAuthenticateUser(self):
 		'''
 		Authenticate a user by Windows-Login on current machine
+
+		:raises BackendAuthenticationError: If authentication fails.
 		'''
 		logger.confidential(u"Trying to authenticate user '%s' with password '%s' by win32security" % (self._username, self._password))
 
 		try:
 			win32security.LogonUser(self._username, 'None', self._password, win32security.LOGON32_LOGON_NETWORK, win32security.LOGON32_PROVIDER_DEFAULT)
 			if self._forceGroups is not None:
-				self._userGroups = self._forceGroups
+				self._userGroups = set(self._forceGroups)
 				logger.info(u"Forced groups for user '%s': %s" % (self._username, self._userGroups))
 			else:
 				gresume = 0
@@ -698,15 +767,14 @@ class BackendAccessControl(object):
 							(users, total, uresume) = win32net.NetLocalGroupGetMembers(None, groupname, 0, uresume)
 							for sid in (u['sid'] for u in users):
 								(username, domain, type) = win32security.LookupAccountSid(None, sid)
-								if (username.lower() == self._username.lower()):
-									self._userGroups.append(groupname)
+								if username.lower() == self._username.lower():
+									self._userGroups.add(groupname)
 									logger.debug(u"User '%s' is member of group '%s'" % (self._username, groupname))
-							if (uresume == 0):
+							if uresume == 0:
 								break
-						if (gresume == 0):
+						if gresume == 0:
 							break
 		except Exception as e:
-			# Something failed => raise authentication error
 			raise BackendAuthenticationError(u"Win32security authentication failed for user '%s': %s" % (self._username, e))
 
 	def _pamAuthenticateUser(self):
@@ -714,6 +782,8 @@ class BackendAccessControl(object):
 		Authenticate a user by PAM (Pluggable Authentication Modules).
 		Important: the uid running this code needs access to /etc/shadow
 		if os uses traditional unix authentication mechanisms.
+
+		:raises BackendAuthenticationError: If authentication fails.
 		'''
 		logger.confidential(u"Trying to authenticate user '%s' with password '%s' by PAM" % (self._username, self._password))
 
@@ -757,18 +827,15 @@ class BackendAccessControl(object):
 			auth.acct_mgmt()
 
 			if self._forceGroups is not None:
-				self._userGroups = self._forceGroups
+				self._userGroups = set(self._forceGroups)
 				logger.info(u"Forced groups for user '%s': %s" % (self._username, self._userGroups))
 			else:
-				self._userGroups = [forceUnicode(grp.getgrgid(pwd.getpwnam(self._username)[3])[0])]
-				logger.debug(u"Primary group of user '%s' is '%s'" % (self._username, self._userGroups[0]))
-				groups = grp.getgrall()
-				for group in groups:
-					if self._username in group[3]:
-						gn = forceUnicode(group[0])
-						if gn not in self._userGroups:
-							self._userGroups.append(gn)
-							logger.debug(u"User '%s' is member of group '%s'" % (self._username, gn))
+				primaryGroup = forceUnicode(grp.getgrgid(pwd.getpwnam(self._username)[3])[0])
+				logger.debug(u"Primary group of user '{0}' is '{1}'".format(self._username, primaryGroup))
+
+				self._userGroups = set(forceUnicode(group[0]) for group in grp.getgrall() if self._username in group[3])
+				self._userGroups.add(primaryGroup)
+				logger.debug(u"User '{0}' is member of groups: {1}".format(self._username, self._userGroups))
 		except Exception as e:
 			raise BackendAuthenticationError(u"PAM authentication failed for user '%s': %s" % (self._username, e))
 
@@ -805,7 +872,7 @@ class BackendAccessControl(object):
 		if not params:
 			return False
 		for (param, value) in params.items():
-			if type(value) is types.ClassType and issubclass(value, Object) and (value.id == self._username):
+			if isinstance(value, types.ClassType) and issubclass(value, Object) and (value.id == self._username):
 				return True
 			if param in ('id', 'objectId', 'hostId', 'clientId', 'serverId', 'depotId') and (value == self._username):
 				return True
@@ -829,17 +896,17 @@ class BackendAccessControl(object):
 				aclType = entry.get('type')
 				ids = entry.get('ids', [])
 				newGranted = False
-				if (aclType == 'all'):
+				if aclType == 'all':
 					newGranted = True
-				elif (aclType == 'opsi_depotserver'):
+				elif aclType == 'opsi_depotserver':
 					newGranted = self._isOpsiDepotserver(ids)
-				elif (aclType == 'opsi_client'):
+				elif aclType == 'opsi_client':
 					newGranted = self._isOpsiClient(ids)
-				elif (aclType == 'sys_group'):
+				elif aclType == 'sys_group':
 					newGranted = self._isMemberOfGroup(ids)
-				elif (aclType == 'sys_user'):
+				elif aclType == 'sys_user':
 					newGranted = self._isUser(ids)
-				elif (aclType == 'self'):
+				elif aclType == 'self':
 					newGranted = 'partial_object'
 				else:
 					logger.error(u"Unhandled acl entry type: %s" % aclType)
@@ -848,19 +915,19 @@ class BackendAccessControl(object):
 				if newGranted is False:
 					continue
 
-				if (entry.get('denyAttributes') or entry.get('allowAttributes')):
+				if entry.get('denyAttributes') or entry.get('allowAttributes'):
 					newGranted = 'partial_attributes'
 
 				if newGranted:
 					acls.append(entry)
 					granted = newGranted
+
 				if granted is True:
 					break
-			# if granted:
 			break
 
-		logger.info("Method: %s, using acls: %s" % (methodName, acls))
-		if   granted is True:
+		logger.debug("Method {0!r} using acls: {1}".format(methodName, acls))
+		if granted is True:
 			logger.debug(u"Full access to method '%s' granted to user '%s' by acl %s" % (methodName, self._username, acls[0]))
 			newKwargs = kwargs
 		elif granted is False:
@@ -891,16 +958,15 @@ class BackendAccessControl(object):
 		params = dict(params)
 		logger.debug(u"Filtering params: %s" % params)
 		for (key, value) in params.items():
-			isList = type(value) is list
 			valueList = forceList(value)
-			if (len(valueList) == 0):
+			if len(valueList) == 0:
 				continue
-			if issubclass(valueList[0].__class__, BaseObject) or type(valueList[0]) is types.DictType:
-				valueList = self._filterObjects(valueList, acls, exceptionOnTruncate = False)
-				if isList:
+			if issubclass(valueList[0].__class__, BaseObject) or isinstance(valueList[0], dict):
+				valueList = self._filterObjects(valueList, acls, exceptionOnTruncate=False)
+				if isinstance(value, list):
 					params[key] = valueList
 				else:
-					if (len(valueList) > 0):
+					if len(valueList) > 0:
 						params[key] = valueList[0]
 					else:
 						del params[key]
@@ -908,14 +974,13 @@ class BackendAccessControl(object):
 
 	def _filterResult(self, result, acls):
 		if result:
-			isList = type(result) is list
 			resultList = forceList(result)
-			if issubclass(resultList[0].__class__, BaseObject) or type(resultList[0]) is types.DictType:
-				resultList = self._filterObjects(result, acls, exceptionOnTruncate = False, exceptionIfAllRemoved = False)
-				if isList:
+			if issubclass(resultList[0].__class__, BaseObject) or isinstance(resultList[0], dict):
+				resultList = self._filterObjects(result, acls, exceptionOnTruncate=False, exceptionIfAllRemoved=False)
+				if isinstance(result, list):
 					return resultList
 				else:
-					if (len(resultList) > 0):
+					if len(resultList) > 0:
 						return resultList[0]
 					else:
 						return None
@@ -925,7 +990,7 @@ class BackendAccessControl(object):
 		logger.info(u"Filtering objects by acls")
 		newObjects = []
 		for obj in forceList(objects):
-			isDict = type(obj) is types.DictType
+			isDict = isinstance(obj, dict)
 			if isDict:
 				objHash = obj
 			else:
@@ -940,7 +1005,7 @@ class BackendAccessControl(object):
 							objectId = objHash[identifier]
 							break
 
-					if not objectId or (objectId != self._username):
+					if not objectId or objectId != self._username:
 						continue
 
 				if acl.get('allowAttributes'):
@@ -983,7 +1048,7 @@ class BackendAccessControl(object):
 def backendManagerFactory(user, password, dispatchConfigFile, backendConfigDir,
 				extensionConfigDir, aclFile, depotId, postpath, context, **kwargs):
 	backendManager = None
-	if (len(postpath) == 2) and (postpath[0] == 'backend'):
+	if len(postpath) == 2 and postpath[0] == 'backend':
 		backendManager = BackendManager(
 			backend=postpath[1],
 			accessControlContext=context,
@@ -993,7 +1058,7 @@ def backendManagerFactory(user, password, dispatchConfigFile, backendConfigDir,
 			password=password,
 			**kwargs
 		)
-	elif (len(postpath) == 2) and (postpath[0] == 'extend'):
+	elif len(postpath) == 2 and postpath[0] == 'extend':
 		extendPath = postpath[1]
 		if not re.search('^[a-zA-Z0-9\_\-]+$', extendPath):
 			raise ValueError(u"Extension config path '%s' refused" % extendPath)
