@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # This file is part of python-opsi.
-# Copyright (C) 2010-2014 uib GmbH <info@uib.de>
+# Copyright (C) 2010-2016 uib GmbH <info@uib.de>
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -19,14 +19,12 @@
 """
 Depotserver backend.
 
-:copyright:	uib GmbH <info@uib.de>
+:copyright: uib GmbH <info@uib.de>
 :author: Jan Schneider <j.schneider@uib.de>
 :author: Niko Wenselowski <n.wenselowski@uib.de>
 :license: GNU Affero General Public License version 3
 """
 
-__version__ = '4.0.6.1'
-
 import os
 
 from OPSI.Logger import Logger
@@ -42,6 +40,8 @@ from OPSI.Util import (compareVersions, getfqdn, md5sum, librsyncSignature,
 	librsyncPatchFile, librsyncDeltaFile, removeDirectory)
 from OPSI.Util.File import ZsyncFile
 
+__version__ = '4.0.6.1'
+
 logger = Logger()
 
 
@@ -55,7 +55,7 @@ class DepotserverBackend(ExtendedBackend):
 		self._sshRSAPublicKeyFile = u'/etc/ssh/ssh_host_rsa_key.pub'
 
 		self._depotId = forceHostId(getfqdn(conf=OPSI_GLOBAL_CONF))
-		if not self._context.host_getIdents(id=self._depotId):
+		if not self._context.host_getIdents(id=self._depotId):  # pylint: disable=maybe-no-member
 			raise BackendMissingDataError(u"Depot '%s' not found in backend" % self._depotId)
 		self._packageManager = DepotserverPackageManager(self)
 
@@ -98,8 +98,8 @@ class DepotserverBackend(ExtendedBackend):
 		except Exception as e:
 			raise BackendIOError(u"Failed to get disk space usage: %s" % e)
 
-	def depot_installPackage(self, filename, force=False, propertyDefaultValues={}, tempDir=None):
-		self._packageManager.installPackage(filename, force, propertyDefaultValues, tempDir)
+	def depot_installPackage(self, filename, force=False, propertyDefaultValues={}, tempDir=None, onlyMetadata=False):
+		self._packageManager.installPackage(filename, force, propertyDefaultValues, tempDir, onlyMetadata)
 
 	def depot_uninstallPackage(self, productId, force=False, deleteFiles=True):
 		self._packageManager.uninstallPackage(productId, force, deleteFiles)
@@ -127,7 +127,7 @@ class DepotserverPackageManager(object):
 		self._depotBackend = depotBackend
 		logger.setLogFile(self._depotBackend._packageLog, object=self)
 
-	def installPackage(self, filename, force=False, propertyDefaultValues={}, tempDir=None):
+	def installPackage(self, filename, force=False, propertyDefaultValues={}, tempDir=None, onlyMetadata=False):
 		depotId = self._depotBackend._depotId
 		logger.notice(u"=================================================================================================")
 		logger.notice(u"Installing package file '%s' on depot '%s'" % (filename, depotId))
@@ -190,18 +190,19 @@ class DepotserverPackageManager(object):
 				logger.notice(u"Checking package dependencies")
 				self.checkDependencies(ppf)
 
-				logger.notice(u"Running preinst script")
-				for line in ppf.runPreinst(({'DEPOT_ID': depotId})):
-					logger.info(u"[preinst] %s" % line)
+				if not onlyMetadata:
+					logger.notice(u"Running preinst script")
+					for line in ppf.runPreinst(({'DEPOT_ID': depotId})):
+						logger.info(u"[preinst] %s" % line)
 
-				logger.notice(u"Unpacking package files")
-				if ppf.packageControlFile.getIncrementalPackage():
-					logger.info(u"Incremental package, not deleting old client-data files")
-				else:
-					logger.info(u"Deleting old client-data dir")
-					ppf.deleteProductClientDataDir()
+					logger.notice(u"Unpacking package files")
+					if ppf.packageControlFile.getIncrementalPackage():
+						logger.info(u"Incremental package, not deleting old client-data files")
+					else:
+						logger.info(u"Deleting old client-data dir")
+						ppf.deleteProductClientDataDir()
 
-				ppf.extractData()
+					ppf.extractData()
 
 				logger.info(u"Updating product dependencies of product %s" % product)
 				currentProductDependencies = {}
@@ -296,12 +297,13 @@ class DepotserverPackageManager(object):
 									% (propertyDefaultValues[productPropertyState.propertyId], productPropertyState, e) )
 				self._depotBackend._context.productPropertyState_createObjects(productPropertyStates)
 
-				logger.notice(u"Running postinst script")
-				for line in ppf.runPostinst({'DEPOT_ID': depotId}):
-					logger.info(u"[postinst] %s" % line)
+				if not onlyMetadata:
+					logger.notice(u"Running postinst script")
+					for line in ppf.runPostinst({'DEPOT_ID': depotId}):
+						logger.info(u"[postinst] %s" % line)
 
-				ppf.createPackageContentFile()
-				ppf.setAccessRights()
+					ppf.createPackageContentFile()
+					ppf.setAccessRights()
 				ppf.cleanup()
 
 				logger.notice(u"Unlocking product '%s_%s-%s' on depot '%s'" \
