@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # This file is part of python-opsi.
-# Copyright (C) 2006-2014 uib GmbH <info@uib.de>
+# Copyright (C) 2006-2016 uib GmbH <info@uib.de>
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -27,8 +27,6 @@ This backend stores all it's data in plaintext files.
 :license: GNU Affero General Public License version 3
 """
 
-__version__ = '4.0.6.1'
-
 import grp
 import os
 import pwd
@@ -45,6 +43,9 @@ from OPSI.Util import toJson, fromJson, getfqdn
 from OPSI.Util.File import IniFile, LockableFile
 from OPSI.Util.File.Opsi import OpsiConfFile, HostKeyFile, PackageControlFile
 from OPSI.Object import *  # needed for calls to "eval"
+from OPSI.Types import BackendConfigurationError
+
+__version__ = '4.0.6.46'
 
 logger = Logger()
 
@@ -63,10 +64,10 @@ class FileBackend(ConfigDataBackend):
 
 		self.__fileUser = u'opsiconfd'
 		self.__fileGroup = u'pcpatch'
-		self.__fileMode = 0660
+		self.__fileMode = 0o660
 		self.__dirGroup = u'pcpatch'
 		self.__dirUser = u'opsiconfd'
-		self.__dirMode = 0770
+		self.__dirMode = 0o770
 
 		try:
 			self.__fileGroup = OpsiConfFile().getOpsiFileAdminGroup()
@@ -187,7 +188,8 @@ class FileBackend(ConfigDataBackend):
 			'ProductOnDepot': [
 				{'fileType': 'ini', 'attribute': 'productType', 'section': '<productId>-state', 'option': 'producttype', 'json': False},
 				{'fileType': 'ini', 'attribute': 'productVersion', 'section': '<productId>-state', 'option': 'productversion', 'json': False},
-				{'fileType': 'ini', 'attribute': 'packageVersion', 'section': '<productId>-state', 'option': 'packageversion', 'json': False}
+				{'fileType': 'ini', 'attribute': 'packageVersion', 'section': '<productId>-state', 'option': 'packageversion', 'json': False},
+				{'fileType': 'ini', 'attribute': 'locked', 'section': '<productId>-state', 'option': 'locked', 'json': False}
 			],
 			'ProductOnClient': [
 				{'fileType': 'ini', 'attribute': 'productType', 'section': '<productId>-state', 'option': 'producttype', 'json': False},
@@ -218,9 +220,7 @@ class FileBackend(ConfigDataBackend):
 		self._mappings['BoolConfig'] = self._mappings['Config']
 		self._mappings['OpsiConfigserver'] = self._mappings['OpsiDepotserver']
 		self._mappings['LocalbootProduct'] = self._mappings['Product']
-		self._mappings['LocalbootProduct'].extend(self._mappings['LocalbootProduct'])
 		self._mappings['NetbootProduct'] = self._mappings['Product']
-		self._mappings['NetbootProduct'].extend(self._mappings['NetbootProduct'])
 		self._mappings['UnicodeProductProperty'] = self._mappings['ProductProperty']
 		self._mappings['BoolProductProperty'] = self._mappings['ProductProperty']
 		self._mappings['HostGroup'] = self._mappings['Group']
@@ -264,24 +264,24 @@ class FileBackend(ConfigDataBackend):
 			os.unlink(self.__productGroupsFile)
 
 	def _setRights(self, path):
-		logger.debug(u"Setting rights for path '%s'" % (path))
+		logger.debug(u"Setting rights for path '{0}'".format(path))
 		try:
 			if os.path.isfile(path):
-				logger.debug(u"Setting rights on file '%s'" % (path))
+				logger.debug(u"Setting rights on file '{0}'".format(path))
 				os.chmod(path, self.__fileMode)
 				if os.geteuid() == 0:
 					os.chown(path, self.__fileUid, self.__fileGid)
 				else:
 					os.chown(path, -1, self.__fileGid)
 			elif os.path.isdir(path):
-				logger.debug(u"Setting rights on directory '%s'" % (path))
+				logger.debug(u"Setting rights on directory '{0}'".format(path))
 				os.chmod(path, self.__dirMode)
 				if os.geteuid() == 0:
 					os.chown(path, self.__dirUid, self.__dirGid)
 				else:
 					os.chown(path, -1, self.__dirGid)
-		except Exception as e:
-			logger.warning(u"Failed to set rights for path '%s': %s" % (path, e))
+		except Exception as error:
+			logger.warning(u"Failed to set rights for path '{0}': {1}".format(path, forceUnicode(error)))
 
 	def _mkdir(self, path):
 		logger.debug(u"Creating path: '%s'" % (path))
@@ -294,7 +294,7 @@ class FileBackend(ConfigDataBackend):
 			f = LockableFile(filename)
 			f.create()
 		else:
-			logger.debug(u"Cannot create existing file, setting rights ..." % ())
+			logger.debug(u"Cannot create existing file, only setting rights.")
 		self._setRights(filename)
 
 	@staticmethod
@@ -436,13 +436,11 @@ class FileBackend(ConfigDataBackend):
 
 					for section in cp.sections():
 						if section.endswith('-state'):
-							objIdents.append(
-								{
+							objIdents.append({
 								'productId': section[:-6],
 								'productType': cp.get(section, 'productType'),
 								'clientId': hostId
-								}
-							)
+							})
 				else:
 					objIdents.append({'id': hostId})
 
@@ -478,15 +476,13 @@ class FileBackend(ConfigDataBackend):
 
 					for section in cp.sections():
 						if section.endswith('-state'):
-							objIdents.append(
-								{
+							objIdents.append({
 								'productId': section[:-6],
 								'productType': cp.get(section, 'producttype'),
 								'productVersion': cp.get(section, 'productversion'),
 								'packageVersion': cp.get(section, 'packageversion'),
 								'depotId': hostId
-								}
-							)
+							})
 				else:
 					objIdents.append({'id': hostId})
 
@@ -544,11 +540,10 @@ class FileBackend(ConfigDataBackend):
 						logger.debug2(u"Ignoring invalid file '%s'" % (filename))
 						continue
 
-					objectId = None
 					try:
 						objectId = forceHostId(entry[:-4])
 					except Exception as e:
-						logger.warning(u"Ignoring invalid file '%s': %s" % filename, e)
+						logger.warning(u"Ignoring invalid file '%s': %s" % filename, forceUnicode(e))
 						continue
 
 					if not self._objectHashMatches({'objectId': objectId}, **filter):
@@ -559,12 +554,10 @@ class FileBackend(ConfigDataBackend):
 
 					if objType == 'ConfigState' and cp.has_section('generalconfig'):
 						for option in cp.options('generalconfig'):
-							objIdents.append(
-								{
+							objIdents.append({
 								'configId': option,
 								'objectId': objectId
-								}
-							)
+							})
 					elif objType == 'ProductPropertyState':
 						for section in cp.sections():
 							if not section.endswith('-install'):
@@ -613,8 +606,7 @@ class FileBackend(ConfigDataBackend):
 							try:
 								value = cp.get(section, option)
 								if not forceBool(value):
-									logger.debug(u"Skipping '%s' in section '%s' with False-value '%s'" \
-										% (option, section, value))
+									logger.debug(u"Skipping '%s' in section '%s' with False-value '%s'" % (option, section, value))
 									continue
 								if groupType == 'HostGroup':
 									option = forceHostId(option)
@@ -629,8 +621,7 @@ class FileBackend(ConfigDataBackend):
 									}
 								)
 							except Exception as e:
-								logger.error(u"Found invalid option '%s' in section '%s' in file '%s': %s" \
-									% (option, section, p['filename'], e))
+								logger.error(u"Found invalid option '%s' in section '%s' in file '%s': %s" % (option, section, p['filename'], forceUnicode(e)))
 					else:
 						objIdents.append({'id': section, 'type': groupType})
 
@@ -719,13 +710,8 @@ class FileBackend(ConfigDataBackend):
 			logger.debug2(u"Returning idents without filter." % ())
 			return objIdents
 
-		filteredObjIdents = []
-		for ident in objIdents:
-			if self._objectHashMatches(ident, **filter):
-				filteredObjIdents.append(ident)
-
-		logger.debug2(u"Returning filtered idents." % ())
-		return filteredObjIdents
+		return [ident for ident in objIdents
+				if self._objectHashMatches(ident, **filter)]
 
 	@staticmethod
 	def _adaptObjectHashAttributes(objHash, ident, attributes):
@@ -773,7 +759,7 @@ class FileBackend(ConfigDataBackend):
 		mappings = {}
 		for mapping in self._mappings[objType]:
 			if (not attributes or mapping['attribute'] in attributes) or mapping['attribute'] in filter.keys():
-				if not mappings.has_key(mapping['fileType']):
+				if mapping['fileType'] not in mappings:
 					mappings[mapping['fileType']] = []
 
 				mappings[mapping['fileType']].append(mapping)
@@ -835,13 +821,13 @@ class FileBackend(ConfigDataBackend):
 
 						match = self._placeholderRegex.search(section)
 						if match:
-							section = u'%s%s%s' % (match.group(1), objHash[match.group(2)], match.group(3))
-							if objType == 'ProductOnClient': #<productType>_product_states
+							section = u'%s%s%s' % (match.group(1), objHash[match.group(2)], match.group(3))  # pylint: disable=maybe-no-member
+							if objType == 'ProductOnClient':  # <productType>_product_states
 								section = section.replace('LocalbootProduct', 'localboot').replace('NetbootProduct', 'netboot')
 
 						match = self._placeholderRegex.search(option)
 						if match:
-							option = u'%s%s%s' % (match.group(1), objHash[match.group(2)], match.group(3))
+							option = u'%s%s%s' % (match.group(1), objHash[match.group(2)], match.group(3))  # pylint: disable=maybe-no-member
 
 						if cp.has_option(section, option):
 							value = cp.get(section, option)
@@ -852,7 +838,7 @@ class FileBackend(ConfigDataBackend):
 
 							# invalid values will throw exceptions later
 							if objType == 'ProductOnClient' and section.endswith('_product_states'):
-								index = value.find(':')
+								index = value.find(':')  # pylint: disable=maybe-no-member
 								if index == -1:
 									raise Exception(u"No ':' found in section '%s' in option '%s' in '%s'" % (section, option, filename))
 
@@ -913,10 +899,9 @@ class FileBackend(ConfigDataBackend):
 
 		if objType == 'OpsiConfigserver':
 			if self.__serverId != obj.getId():
-				raise Exception(u"Filebackend can only handle this config server '%s', not '%s'" \
-					% (self.__serverId, obj.getId()))
+				raise Exception(u"Filebackend can only handle this config server '%s', not '%s'" % (self.__serverId, obj.getId()))
 
-		if not self._mappings.has_key(objType):
+		if objType not in self._mappings:
 			raise Exception(u"Mapping not found for object type '%s'" % objType)
 
 		mappings = {}
@@ -963,8 +948,6 @@ class FileBackend(ConfigDataBackend):
 							removeOptions[obj.getId()].append(m['option'])
 					elif objType in ('ProductOnDepot', 'ProductOnClient'):
 						removeSections = [obj.getProductId() + u'-state']
-					elif objType == 'ProductPropertyState':
-						removeSections = [obj.getPropertyId() + u'-install']
 
 					for section in removeSections:
 						if cp.has_section(section):
@@ -1137,7 +1120,7 @@ class FileBackend(ConfigDataBackend):
 		elif objType in ('Product', 'LocalbootProduct', 'NetbootProduct'):
 			for obj in objList:
 				filename = self._getConfigFile(
-					obj.getType(), obj.getIdent(returnType='dict'), 'pro' )
+					obj.getType(), obj.getIdent(returnType='dict'), 'pro')
 				logger.debug(u"Deleting %s: '%s'" % (obj.getType(), obj.getIdent()))
 				if os.path.isfile(filename):
 					os.unlink(filename)
@@ -1222,7 +1205,7 @@ class FileBackend(ConfigDataBackend):
 				for obj in objList:
 					section = None
 					if obj.getType() == 'ObjectToGroup':
-						if not obj.groupType in ('HostGroup', 'ProductGroup'):
+						if obj.groupType not in ('HostGroup', 'ProductGroup'):
 							raise BackendBadValueError(u"Unhandled group type '%s'" % obj.groupType)
 						if not groupType == obj.groupType:
 							continue
@@ -1236,8 +1219,7 @@ class FileBackend(ConfigDataBackend):
 					if obj.getType() == 'ObjectToGroup':
 						if cp.has_option(section, obj.getObjectId()):
 							cp.remove_option(section, obj.getObjectId())
-							logger.debug2(u"Removed option '%s' in section '%s'" \
-								% (obj.getObjectId(), section))
+							logger.debug2(u"Removed option '%s' in section '%s'" % (obj.getObjectId(), section))
 					else:
 						if cp.has_section(section):
 							cp.remove_section(section)
@@ -1255,14 +1237,14 @@ class FileBackend(ConfigDataBackend):
 		host = forceObjectClass(host, Host)
 		ConfigDataBackend.host_insertObject(self, host)
 
-		logger.debug(u"Inserting host: '%s'" % host.getIdent())
+		logger.debug(u"Inserting host: '%s'" % host.getIdent())  # pylint: disable=maybe-no-member
 		self._write(host, mode='create')
 
 	def host_updateObject(self, host):
 		host = forceObjectClass(host, Host)
 		ConfigDataBackend.host_updateObject(self, host)
 
-		logger.debug(u"Updating host: '%s'" % host.getIdent())
+		logger.debug(u"Updating host: '%s'" % host.getIdent())  # pylint: disable=maybe-no-member
 		self._write(host, mode='update')
 
 	def host_getObjects(self, attributes=[], **filter):
@@ -1297,14 +1279,14 @@ class FileBackend(ConfigDataBackend):
 		config = forceObjectClass(config, Config)
 		ConfigDataBackend.config_insertObject(self, config)
 
-		logger.debug(u"Inserting config: '%s'" % config.getIdent())
+		logger.debug(u"Inserting config: '%s'" % config.getIdent())  # pylint: disable=maybe-no-member
 		self._write(config, mode='create')
 
 	def config_updateObject(self, config):
 		config = forceObjectClass(config, Config)
 		ConfigDataBackend.config_updateObject(self, config)
 
-		logger.debug(u"Updating config: '%s'" % config.getIdent())
+		logger.debug(u"Updating config: '%s'" % config.getIdent())  # pylint: disable=maybe-no-member
 		self._write(config, mode='update')
 
 	def config_getObjects(self, attributes=[], **filter):
@@ -1324,14 +1306,14 @@ class FileBackend(ConfigDataBackend):
 		configState = forceObjectClass(configState, ConfigState)
 		ConfigDataBackend.configState_insertObject(self, configState)
 
-		logger.debug(u"Inserting configState: '%s'" % configState.getIdent())
+		logger.debug(u"Inserting configState: '%s'" % configState.getIdent())  # pylint: disable=maybe-no-member
 		self._write(configState, mode='create')
 
 	def configState_updateObject(self, configState):
 		configState = forceObjectClass(configState, ConfigState)
 		ConfigDataBackend.configState_updateObject(self, configState)
 
-		logger.debug(u"Updating configState: '%s'" % configState.getIdent())
+		logger.debug(u"Updating configState: '%s'" % configState.getIdent())  # pylint: disable=maybe-no-member
 		self._write(configState, mode='update')
 
 	def configState_getObjects(self, attributes=[], **filter):
@@ -1351,14 +1333,14 @@ class FileBackend(ConfigDataBackend):
 		product = forceObjectClass(product, Product)
 		ConfigDataBackend.product_insertObject(self, product)
 
-		logger.debug(u"Inserting product: '%s'" % product.getIdent())
+		logger.debug(u"Inserting product: '%s'" % product.getIdent())  # pylint: disable=maybe-no-member
 		self._write(product, mode='create')
 
 	def product_updateObject(self, product):
 		product = forceObjectClass(product, Product)
 		ConfigDataBackend.product_updateObject(self, product)
 
-		logger.debug(u"Updating product: '%s'" % product.getIdent())
+		logger.debug(u"Updating product: '%s'" % product.getIdent())  # pylint: disable=maybe-no-member
 		self._write(product, mode='update')
 
 	def product_getObjects(self, attributes=[], **filter):
@@ -1381,14 +1363,14 @@ class FileBackend(ConfigDataBackend):
 		productProperty = forceObjectClass(productProperty, ProductProperty)
 		ConfigDataBackend.productProperty_insertObject(self, productProperty)
 
-		logger.debug(u"Inserting productProperty: '%s'" % productProperty.getIdent())
+		logger.debug(u"Inserting productProperty: '%s'" % productProperty.getIdent())  # pylint: disable=maybe-no-member
 		self._write(productProperty, mode='create')
 
 	def productProperty_updateObject(self, productProperty):
 		productProperty = forceObjectClass(productProperty, ProductProperty)
 		ConfigDataBackend.productProperty_updateObject(self, productProperty)
 
-		logger.debug(u"Updating productProperty: '%s'" % productProperty.getIdent())
+		logger.debug(u"Updating productProperty: '%s'" % productProperty.getIdent())  # pylint: disable=maybe-no-member
 		self._write(productProperty, mode='update')
 
 	def productProperty_getObjects(self, attributes=[], **filter):
@@ -1408,14 +1390,14 @@ class FileBackend(ConfigDataBackend):
 		productDependency = forceObjectClass(productDependency, ProductDependency)
 		ConfigDataBackend.productDependency_insertObject(self, productDependency)
 
-		logger.debug(u"Inserting productDependency: '%s'" % productDependency.getIdent())
+		logger.debug(u"Inserting productDependency: '%s'" % productDependency.getIdent())  # pylint: disable=maybe-no-member
 		self._write(productDependency, mode='create')
 
 	def productDependency_updateObject(self, productDependency):
 		productDependency = forceObjectClass(productDependency, ProductDependency)
 		ConfigDataBackend.productDependency_updateObject(self, productDependency)
 
-		logger.debug(u"Updating productDependency: '%s'" % productDependency.getIdent())
+		logger.debug(u"Updating productDependency: '%s'" % productDependency.getIdent())  # pylint: disable=maybe-no-member
 		self._write(productDependency, mode='update')
 
 	def productDependency_getObjects(self, attributes=[], **filter):
@@ -1435,14 +1417,14 @@ class FileBackend(ConfigDataBackend):
 		productOnDepot = forceObjectClass(productOnDepot, ProductOnDepot)
 		ConfigDataBackend.productOnDepot_insertObject(self, productOnDepot)
 
-		logger.debug(u"Inserting productOnDepot: '%s'" % productOnDepot.getIdent())
+		logger.debug(u"Inserting productOnDepot: '%s'" % productOnDepot.getIdent())  # pylint: disable=maybe-no-member
 		self._write(productOnDepot, mode='create')
 
 	def productOnDepot_updateObject(self, productOnDepot):
 		productOnDepot = forceObjectClass(productOnDepot, ProductOnDepot)
 		ConfigDataBackend.productOnDepot_updateObject(self, productOnDepot)
 
-		logger.debug(u"Updating productOnDepot: '%s'" % productOnDepot.getIdent())
+		logger.debug(u"Updating productOnDepot: '%s'" % productOnDepot.getIdent())  # pylint: disable=maybe-no-member
 		self._write(productOnDepot, mode='update')
 
 	def productOnDepot_getObjects(self, attributes=[], **filter):
@@ -1462,14 +1444,14 @@ class FileBackend(ConfigDataBackend):
 		productOnClient = forceObjectClass(productOnClient, ProductOnClient)
 		ConfigDataBackend.productOnClient_insertObject(self, productOnClient)
 
-		logger.debug(u"Inserting productOnClient: '%s'" % productOnClient.getIdent())
+		logger.debug(u"Inserting productOnClient: '%s'" % productOnClient.getIdent())  # pylint: disable=maybe-no-member
 		self._write(productOnClient, mode='create')
 
 	def productOnClient_updateObject(self, productOnClient):
 		productOnClient = forceObjectClass(productOnClient, ProductOnClient)
 		ConfigDataBackend.productOnClient_updateObject(self, productOnClient)
 
-		logger.debug(u"Updating productOnClient: '%s'" % productOnClient.getIdent())
+		logger.debug(u"Updating productOnClient: '%s'" % productOnClient.getIdent())  # pylint: disable=maybe-no-member
 		self._write(productOnClient, mode='update')
 
 	def productOnClient_getObjects(self, attributes=[], **filter):
@@ -1489,14 +1471,14 @@ class FileBackend(ConfigDataBackend):
 		productPropertyState = forceObjectClass(productPropertyState, ProductPropertyState)
 		ConfigDataBackend.productPropertyState_insertObject(self, productPropertyState)
 
-		logger.debug(u"Inserting productPropertyState: '%s'" % productPropertyState.getIdent())
+		logger.debug(u"Inserting productPropertyState: '%s'" % productPropertyState.getIdent())  # pylint: disable=maybe-no-member
 		self._write(productPropertyState, mode='create')
 
 	def productPropertyState_updateObject(self, productPropertyState):
 		productPropertyState = forceObjectClass(productPropertyState, ProductPropertyState)
 		ConfigDataBackend.productPropertyState_updateObject(self, productPropertyState)
 
-		logger.debug(u"Updating productPropertyState: '%s'" % productPropertyState.getIdent())
+		logger.debug(u"Updating productPropertyState: '%s'" % productPropertyState.getIdent())  # pylint: disable=maybe-no-member
 		self._write(productPropertyState, mode='update')
 
 	def productPropertyState_getObjects(self, attributes=[], **filter):
@@ -1516,14 +1498,14 @@ class FileBackend(ConfigDataBackend):
 		group = forceObjectClass(group, Group)
 		ConfigDataBackend.group_insertObject(self, group)
 
-		logger.debug(u"Inserting group: '%s'" % group.getIdent())
+		logger.debug(u"Inserting group: '%s'" % group.getIdent())  # pylint: disable=maybe-no-member
 		self._write(group, mode='create')
 
 	def group_updateObject(self, group):
 		group = forceObjectClass(group, Group)
 		ConfigDataBackend.group_updateObject(self, group)
 
-		logger.debug(u"Updating group: '%s'" % group.getIdent())
+		logger.debug(u"Updating group: '%s'" % group.getIdent())  # pylint: disable=maybe-no-member
 		self._write(group, mode='update')
 
 	def group_getObjects(self, attributes=[], **filter):
@@ -1543,14 +1525,14 @@ class FileBackend(ConfigDataBackend):
 		objectToGroup = forceObjectClass(objectToGroup, ObjectToGroup)
 		ConfigDataBackend.objectToGroup_insertObject(self, objectToGroup)
 
-		logger.debug(u"Inserting objectToGroup: '%s'" % objectToGroup.getIdent())
+		logger.debug(u"Inserting objectToGroup: '%s'" % objectToGroup.getIdent())  # pylint: disable=maybe-no-member
 		self._write(objectToGroup, mode='create')
 
 	def objectToGroup_updateObject(self, objectToGroup):
 		objectToGroup = forceObjectClass(objectToGroup, ObjectToGroup)
 		ConfigDataBackend.objectToGroup_updateObject(self, objectToGroup)
 
-		logger.debug(u"Updating objectToGroup: '%s'" % objectToGroup.getIdent())
+		logger.debug(u"Updating objectToGroup: '%s'" % objectToGroup.getIdent())  # pylint: disable=maybe-no-member
 		self._write(objectToGroup, mode='update')
 
 	def objectToGroup_getObjects(self, attributes=[], **filter):
@@ -1570,7 +1552,7 @@ class FileBackend(ConfigDataBackend):
 		auditSoftware = forceObjectClass(auditSoftware, AuditSoftware)
 		ConfigDataBackend.auditSoftware_insertObject(self, auditSoftware)
 
-		logger.debug(u"Inserting auditSoftware: '%s'" % auditSoftware.getIdent())
+		logger.debug(u"Inserting auditSoftware: '%s'" % auditSoftware.getIdent())  # pylint: disable=maybe-no-member
 		filename = self._getConfigFile('AuditSoftware', {}, 'sw')
 
 		if not os.path.exists(filename):
@@ -1579,7 +1561,7 @@ class FileBackend(ConfigDataBackend):
 		iniFile = IniFile(filename=filename)
 		ini = iniFile.parse()
 
-		auditSoftware = auditSoftware.toHash()
+		auditSoftware = auditSoftware.toHash()  # pylint: disable=maybe-no-member
 		for attribute in auditSoftware.keys():
 			if (auditSoftware[attribute] is None) or (attribute == 'type'):
 				continue
@@ -1625,11 +1607,11 @@ class FileBackend(ConfigDataBackend):
 		auditSoftware = forceObjectClass(auditSoftware, AuditSoftware)
 		ConfigDataBackend.auditSoftware_updateObject(self, auditSoftware)
 
-		logger.debug(u"Updating auditSoftware: '%s'" % auditSoftware.getIdent())
+		logger.debug(u"Updating auditSoftware: '%s'" % auditSoftware.getIdent())  # pylint: disable=maybe-no-member
 		filename = self._getConfigFile('AuditSoftware', {}, 'sw')
 		iniFile = IniFile(filename=filename)
 		ini = iniFile.parse()
-		ident = auditSoftware.getIdent(returnType='dict')
+		ident = auditSoftware.getIdent(returnType='dict')  # pylint: disable=maybe-no-member
 
 		for section in ini.sections():
 			found = True
@@ -1639,7 +1621,7 @@ class FileBackend(ConfigDataBackend):
 					break
 
 			if found:
-				for (key, value) in auditSoftware.toHash().items():
+				for (key, value) in auditSoftware.toHash().items():  # pylint: disable=maybe-no-member
 					if value is None:
 						continue
 					ini.set(section, key, self.__escape(value))
@@ -1683,7 +1665,7 @@ class FileBackend(ConfigDataBackend):
 			for (key, value) in objHash.items():
 				try:
 					value = self.__unescape(ini.get(section, key.lower()))
-					if fastFilter and value and fastFilter.has_key(key) and (fastFilter[key] != value):
+					if fastFilter and value and key in fastFilter and (fastFilter[key] != value):
 						fastFiltered = True
 						break
 					objHash[key] = value
@@ -1704,8 +1686,10 @@ class FileBackend(ConfigDataBackend):
 			return
 		iniFile = IniFile(filename=filename)
 		ini = iniFile.parse()
-		idents = [auditSoftware.getIdent(returnType='dict') for auditSoftware
-				  in forceObjectClassList(auditSoftwares, AuditSoftware)]
+		idents = [
+			auditSoftware.getIdent(returnType='dict') for auditSoftware
+			in forceObjectClassList(auditSoftwares, AuditSoftware)
+		]
 
 		removeSections = []
 		for section in ini.sections():
@@ -1716,7 +1700,7 @@ class FileBackend(ConfigDataBackend):
 						found = False
 						break
 
-				if found and not section in removeSections:
+				if found and section not in removeSections:
 					removeSections.append(section)
 
 		if removeSections:
@@ -1729,8 +1713,8 @@ class FileBackend(ConfigDataBackend):
 		auditSoftwareOnClient = forceObjectClass(auditSoftwareOnClient, AuditSoftwareOnClient)
 		ConfigDataBackend.auditSoftwareOnClient_insertObject(self, auditSoftwareOnClient)
 
-		logger.debug(u"Inserting auditSoftwareOnClient: '%s'" % auditSoftwareOnClient.getIdent())
-		filename = self._getConfigFile('AuditSoftwareOnClient', {"clientId": auditSoftwareOnClient.clientId}, 'sw')
+		logger.debug(u"Inserting auditSoftwareOnClient: '%s'" % auditSoftwareOnClient.getIdent())  # pylint: disable=maybe-no-member
+		filename = self._getConfigFile('AuditSoftwareOnClient', {"clientId": auditSoftwareOnClient.clientId}, 'sw')  # pylint: disable=maybe-no-member
 
 		if not os.path.exists(filename):
 			self._touch(filename)
@@ -1738,7 +1722,7 @@ class FileBackend(ConfigDataBackend):
 		iniFile = IniFile(filename=filename)
 		ini = iniFile.parse()
 
-		auditSoftwareOnClient = auditSoftwareOnClient.toHash()
+		auditSoftwareOnClient = auditSoftwareOnClient.toHash()  # pylint: disable=maybe-no-member
 		for attribute in auditSoftwareOnClient.keys():
 			if auditSoftwareOnClient[attribute] is None:
 				continue
@@ -1783,11 +1767,11 @@ class FileBackend(ConfigDataBackend):
 		auditSoftwareOnClient = forceObjectClass(auditSoftwareOnClient, AuditSoftwareOnClient)
 		ConfigDataBackend.auditSoftwareOnClient_updateObject(self, auditSoftwareOnClient)
 
-		logger.debug(u"Updating auditSoftwareOnClient: '%s'" % auditSoftwareOnClient.getIdent())
-		filename = self._getConfigFile('AuditSoftwareOnClient', {"clientId": auditSoftwareOnClient.clientId}, 'sw')
+		logger.debug(u"Updating auditSoftwareOnClient: '%s'" % auditSoftwareOnClient.getIdent())  # pylint: disable=maybe-no-member
+		filename = self._getConfigFile('AuditSoftwareOnClient', {"clientId": auditSoftwareOnClient.clientId}, 'sw')  # pylint: disable=maybe-no-member
 		iniFile = IniFile(filename=filename)
 		ini = iniFile.parse()
-		ident = auditSoftwareOnClient.getIdent(returnType='dict')
+		ident = auditSoftwareOnClient.getIdent(returnType='dict')  # pylint: disable=maybe-no-member
 
 		for section in ini.sections():
 			found = True
@@ -1797,7 +1781,7 @@ class FileBackend(ConfigDataBackend):
 					break
 
 			if found:
-				for (key, value) in auditSoftwareOnClient.toHash().items():
+				for (key, value) in auditSoftwareOnClient.toHash().items():  # pylint: disable=maybe-no-member
 					if value is None:
 						continue
 					ini.set(section, key, self.__escape(value))
@@ -1875,7 +1859,7 @@ class FileBackend(ConfigDataBackend):
 						if self.__unescape(ini.get(section, key.lower())) != value:
 							found = False
 							break
-					if found and not section in removeSections:
+					if found and section not in removeSections:
 						removeSections.append(section)
 
 			if removeSections:
@@ -1888,14 +1872,14 @@ class FileBackend(ConfigDataBackend):
 		auditHardware = forceObjectClass(auditHardware, AuditHardware)
 		ConfigDataBackend.auditHardware_insertObject(self, auditHardware)
 
-		logger.debug(u"Inserting auditHardware: '%s'" % auditHardware.getIdent())
+		logger.debug(u"Inserting auditHardware: '%s'" % auditHardware.getIdent())  # pylint: disable=maybe-no-member
 		self.__doAuditHardwareObj(auditHardware, mode='insert')
 
 	def auditHardware_updateObject(self, auditHardware):
 		auditHardware = forceObjectClass(auditHardware, AuditHardware)
 		ConfigDataBackend.auditHardware_updateObject(self, auditHardware)
 
-		logger.debug(u"Updating auditHardware: '%s'" % auditHardware.getIdent())
+		logger.debug(u"Updating auditHardware: '%s'" % auditHardware.getIdent())  # pylint: disable=maybe-no-member
 		self.__doAuditHardwareObj(auditHardware, mode='update')
 
 	def auditHardware_getObjects(self, attributes=[], **filter):
