@@ -1,47 +1,70 @@
-#!/usr/bin/python
+#! /usr/bin/env python
 # -*- coding: utf-8 -*-
+
+# This file is part of python-opsi.
+# Copyright (C) 2006-2015 uib GmbH <info@uib.de>
+
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
 opsi python library - Logger
 
-This module is part of the desktop management solution opsi
-(open pc server integration) http://www.opsi.org
-
-Copyright (C) 2006 - 2013 uib GmbH
-
-http://www.uib.de/
-
-All rights reserved.
-
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License version 2 as
-published by the Free Software Foundation.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-@copyright:	uib GmbH <info@uib.de>
-@author: Jan Schneider <j.schneider@uib.de>
-@license: GNU General Public License version 2
+:copyright: uib GmbH <info@uib.de>
+:author: Jan Schneider <j.schneider@uib.de>
+:author: Niko Wenselowski <n.wenselowski@uib.de>
+:license: GNU Affero General Public License version 3
 """
 
-__version__ = '4.0'
-
-import sys
-import locale
-import time
-import os
-import thread
-import threading
 import codecs
-import types
+import locale
+import os
+import sys
+import threading
+import time
 import warnings
 
+try:
+	import thread
+except ImportError:
+	# Python 3
+	import _thread as thread
+
+if os.name == 'nt':
+	# Windows imports for file locking
+	import win32con
+	import win32file
+	import pywintypes
+elif os.name == 'posix':
+	# Posix imports for file locking
+	import fcntl
+
+try:
+	import syslog
+except ImportError:
+	syslog = None
+
+__version__ = '4.0.6.36'
+
+if sys.version_info > (3, ):
+	# Python 3
+	unicode = str
+	_STRING_TYPE = str
+	_UNICODE_TYPE = str
+else:
+	# Python 2
+	_STRING_TYPE = str
+	_UNICODE_TYPE = unicode
+
 # Loglevels
 LOG_CONFIDENTIAL = 9
 LOG_DEBUG2 = 8
@@ -55,13 +78,25 @@ LOG_ESSENTIAL = 1
 LOG_COMMENT = LOG_ESSENTIAL
 LOG_NONE = 0
 
-if os.name == 'nt':
-	# Windows imports for file locking
-	import win32con
-	import win32file
-	import pywintypes
-
-	# Colors
+if os.name == 'posix':
+	COLOR_NORMAL = '\033[0;0;0m'
+	COLOR_BLACK = '\033[0;30;40m'
+	COLOR_RED = '\033[0;31;40m'
+	COLOR_GREEN = '\033[0;32;40m'
+	COLOR_YELLOW = '\033[0;33;40m'
+	COLOR_BLUE = '\033[0;34;40m'
+	COLOR_MAGENTA = '\033[0;35;40m'
+	COLOR_CYAN = '\033[0;36;40m'
+	COLOR_WHITE = '\033[0;37;40m'
+	COLOR_LIGHT_BLACK = '\033[1;30;40m'
+	COLOR_LIGHT_RED = '\033[1;31;40m'
+	COLOR_LIGHT_GREEN = '\033[1;32;40m'
+	COLOR_LIGHT_YELLOW = '\033[1;33;40m'
+	COLOR_LIGHT_BLUE = '\033[1;34;40m'
+	COLOR_LIGHT_MAGENTA = '\033[1;35;40m'
+	COLOR_LIGHT_CYAN = '\033[1;36;40m'
+	COLOR_LIGHT_WHITE = '\033[1;37;40m'
+else:
 	COLOR_NORMAL = ''
 	COLOR_BLACK = ''
 	COLOR_RED = ''
@@ -80,29 +115,6 @@ if os.name == 'nt':
 	COLOR_LIGHT_CYAN = ''
 	COLOR_LIGHT_WHITE = ''
 
-elif (os.name == 'posix'):
-	# Posix imports for file locking
-	import fcntl
-
-	# Colors
-	COLOR_NORMAL = '\033[0;0;0m'
-	COLOR_BLACK = '\033[0;30;40m'
-	COLOR_RED = '\033[0;31;40m'
-	COLOR_GREEN = '\033[0;32;40m'
-	COLOR_YELLOW = '\033[0;33;40m'
-	COLOR_BLUE = '\033[0;34;40m'
-	COLOR_MAGENTA = '\033[0;35;40m'
-	COLOR_CYAN = '\033[0;36;40m'
-	COLOR_WHITE = '\033[0;37;40m'
-	COLOR_LIGHT_BLACK = '\033[1;30;40m'
-	COLOR_LIGHT_RED = '\033[1;31;40m'
-	COLOR_LIGHT_GREEN = '\033[1;32;40m'
-	COLOR_LIGHT_YELLOW = '\033[1;33;40m'
-	COLOR_LIGHT_BLUE = '\033[1;34;40m'
-	COLOR_LIGHT_MAGENTA = '\033[1;35;40m'
-	COLOR_LIGHT_CYAN = '\033[1;36;40m'
-	COLOR_LIGHT_WHITE = '\033[1;37;40m'
-
 COLORS_AVAILABLE = [
 	COLOR_NORMAL, COLOR_BLACK, COLOR_RED, COLOR_GREEN, COLOR_YELLOW,
 	COLOR_BLUE, COLOR_MAGENTA, COLOR_CYAN, COLOR_WHITE, COLOR_LIGHT_BLACK,
@@ -120,34 +132,48 @@ CONFIDENTIAL_COLOR = COLOR_LIGHT_YELLOW
 ESSENTIAL_COLOR = COLOR_LIGHT_CYAN
 COMMENT_COLOR = ESSENTIAL_COLOR
 
+if syslog is not None:
+	_SYSLOG_LEVEL_MAPPING = {
+		LOG_CONFIDENTIAL: syslog.LOG_DEBUG,
+		LOG_DEBUG2: syslog.LOG_DEBUG,
+		LOG_DEBUG: syslog.LOG_DEBUG,
+		LOG_INFO: syslog.LOG_INFO,
+		LOG_NOTICE: syslog.LOG_NOTICE,
+		LOG_WARNING: syslog.LOG_WARNING,
+		LOG_ERROR: syslog.LOG_ERR,
+		LOG_CRITICAL: syslog.LOG_CRIT,
+		LOG_COMMENT: syslog.LOG_CRIT
+	}
+
 encoding = sys.getfilesystemencoding()
 _showwarning = warnings.showwarning
 
 
 def forceUnicode(var):
-	if type(var) is types.UnicodeType:
+	if isinstance(var, _UNICODE_TYPE):
 		return var
-	elif type(var) is types.StringType:
+	elif isinstance(var, _STRING_TYPE):
 		return unicode(var, 'utf-8', 'replace')
-	elif os.name == 'nt' and type(var) is WindowsError:
+	elif os.name == 'nt' and isinstance(var, WindowsError):
 		return u"[Error %s] %s" % (var.args[0], var.args[1].decode(encoding))
 
-	if hasattr(var, '__unicode__'):
-		try:
-			return var.__unicode__()
-		except:
-			pass
+	try:
+		return var.__unicode__()
+	except Exception:
+		pass
 
 	try:
 		return unicode(var)
 	except Exception:
 		pass
 
-	if hasattr(var, '__repr__'):
+	try:
 		var = var.__repr__()
-		if type(var) is types.UnicodeType:
+		if isinstance(var, _UNICODE_TYPE):
 			return var
 		return unicode(var, 'utf-8', 'replace')
+	except Exception:
+		pass
 
 	return unicode(var)
 
@@ -180,7 +206,7 @@ class LoggerSubject:
 		return self._severity
 
 	def attachObserver(self, observer):
-		if not observer in self._observers:
+		if observer not in self._observers:
 			self._observers.append(observer)
 
 	def detachObserver(self, observer):
@@ -206,8 +232,9 @@ class TwistedLogObserver(object):
 			if eventDict.get('failure'):
 				self._logger.logTraceback(eventDict['failure'].getTracebackObject())
 				self._logger.critical(u"     ==>>> %s" % eventDict['failure'].getErrorMessage())
+
 			for line in eventDict.get('message', ()):
-				if line.find("Can't find property") != -1:
+				if "Can't find property" in line:
 					# Dav property errors
 					self._logger.debug(u"[twisted] %s" % line)
 				else:
@@ -236,7 +263,7 @@ class LoggerImplementation:
 		self.univentionLogger_priv = None
 		self.__univentionClass = None
 		self.__univentionFormat = u'opsi: %M'
-		self.__confidentialStrings = []
+		self.__confidentialStrings = set()
 		self.__componentName = u''
 		self.__threadConfig = {}
 		self.__objectConfig = {}
@@ -252,9 +279,10 @@ class LoggerImplementation:
 		return self.__stdout
 
 	def setConfidentialStrings(self, strings):
-		if not type(strings) in (list, tuple):
+		if not isinstance(strings, (list, tuple)):
 			strings = [strings]
-		self.__confidentialStrings = []
+
+		self.__confidentialStrings = set()
 		for string in strings:
 			self.addConfidentialString(string)
 
@@ -262,9 +290,7 @@ class LoggerImplementation:
 		string = forceUnicode(string)
 		if not string:
 			raise ValueError(u"Cannot use empty string as confidential string")
-		if string in self.__confidentialStrings:
-			return
-		self.__confidentialStrings.append(string)
+		self.__confidentialStrings.add(string)
 
 	def setLogFormat(self, format, currentThread=False, object=None):
 		self.setConsoleFormat(format, currentThread, object)
@@ -347,18 +373,11 @@ class LoggerImplementation:
 		''' Maximum level of messages to log by syslog.
 		Set LOG_NONE to disable syslog (default)'''
 
-		if level < LOG_NONE:
-			level = LOG_NONE
-		if level > LOG_CONFIDENTIAL:
-			level = LOG_CONFIDENTIAL
+		level = self._sanitizeLogLevel(level)
 
 		self.__syslogLevel = level
-		if os.name == 'posix':
+		if syslog is not None:
 			if self.__syslogLevel != LOG_NONE:
-				# Import syslog module
-				global syslog
-				import syslog
-
 				# Set ident string for syslog
 				ident = 'opsi'
 				try:
@@ -369,11 +388,7 @@ class LoggerImplementation:
 					# Get caller's filename
 					filename = frame.f_code.co_filename
 					ident = filename.split('/')[-1]
-				# syslog.openlog(ident, syslog.LOG_PID | syslog.LOG_CONS, syslog.LOG_DAEMON)
 				syslog.openlog(ident, syslog.LOG_CONS, syslog.LOG_DAEMON)
-		else:
-			# TODO: not yet implemented
-			pass
 
 	def setMessageSubjectLevel(self, level=LOG_NONE):
 		self.__messageSubjectLevel = self._sanitizeLogLevel(level)
@@ -407,7 +422,15 @@ class LoggerImplementation:
 		return self.__logFile
 
 	def setLogFile(self, logFile, currentThread=False, object=None):
-		''' Set the filename of logfile. '''
+		'''
+		Set the filename of logfile.
+
+		:param logFile: The path to the logfile. Setting this to `None` \
+will disable logging to a file.
+		'''
+		if logFile is not None:
+			logFile = os.path.abspath(logFile)
+
 		if currentThread:
 			self._setThreadConfig('logFile', logFile)
 			self.debug(u"Now using log-file '%s' for thread %s" \
@@ -439,6 +462,7 @@ class LoggerImplementation:
 		try:
 			if logFile == linkFile:
 				raise Exception(u'logFile and linkFile are the same file!')
+
 			if os.path.exists(linkFile):
 				os.unlink(linkFile)
 			os.symlink(logFile, linkFile)
@@ -455,6 +479,7 @@ class LoggerImplementation:
 			if id(object) in self.__objectConfig:
 				self.debug(u"Deleting config of object 0x%x" % id(object))
 				del self.__objectConfig[id(object)]
+
 			for objectId in self.__objectConfig.keys():
 				self.debug2(u"Got special config for object 0x%x" % objectId)
 
@@ -462,57 +487,81 @@ class LoggerImplementation:
 		if threadId in self.__threadConfig:
 			self.debug(u"Deleting config of thread %s" % threadId)
 			del self.__threadConfig[threadId]
+
 		for threadId in self.__threadConfig.keys():
 			self.debug2(u"Got special config for thread %s" % threadId)
 
 	def _setThreadConfig(self, key, value):
 		threadId = unicode(thread.get_ident())
-		if threadId not in self.__threadConfig:
-			self.__threadConfig[threadId] = {}
-		self.__threadConfig[threadId][key] = value
+
+		try:
+			self.__threadConfig[threadId][key] = value
+		except KeyError:
+			self.__threadConfig[threadId] = {key: value}
 
 	def _getThreadConfig(self, key=None):
 		threadId = unicode(thread.get_ident())
 		if threadId not in self.__threadConfig:
 			return None
+
 		if not key:
 			return self.__threadConfig[threadId]
+
 		return self.__threadConfig[threadId].get(key)
 
 	def _setObjectConfig(self, objectId, key, value):
-		if objectId not in self.__objectConfig:
-			self.__objectConfig[objectId] = {}
-		self.__objectConfig[objectId][key] = value
+		try:
+			self.__objectConfig[objectId][key] = value
+		except KeyError:
+			self.__objectConfig[objectId] = {key: value}
 
 	def _getObjectConfig(self, objectId, key=None):
 		if objectId not in self.__objectConfig:
 			return None
+
 		if not key:
 			return self.__objectConfig[objectId]
+
 		return self.__objectConfig[objectId].get(key)
 
 	def log(self, level, message, raiseException=False):
-		''' Log a message '''
+		'''
+		Log a message with the given level.
+
+		:param level: The log level of this message.
+		:param message: The message to log.
+		:param raiseException: True raises an exception if any error occurs. \
+False suppresses exceptions.
+		:type raiseException: bool
+		'''
+		def formatMessage(unformattedMessage):
+			tempMessage = unicode(unformattedMessage)
+			tempMessage = tempMessage.replace(u'%D', datetime)
+			tempMessage = tempMessage.replace(u'%T', threadId)
+			tempMessage = tempMessage.replace(u'%l', unicode(level))
+			tempMessage = tempMessage.replace(u'%L', levelname)
+			tempMessage = tempMessage.replace(u'%C', componentname)
+			tempMessage = tempMessage.replace(u'%M', message)
+			tempMessage = tempMessage.replace(u'%F', filename)
+			tempMessage = tempMessage.replace(u'%N', linenumber)
+			return tempMessage
+
+		if (level > self.__messageSubjectLevel and
+			level > self.__consoleLevel and
+			level > self.__fileLevel and
+			level > self.__syslogLevel and
+			not self.univentionLogger_priv):
+
+			return
+
 		try:
-			if (level > self.__messageSubjectLevel and
-				level > self.__consoleLevel and
-				level > self.__fileLevel and
-				level > self.__syslogLevel and
-				not self.univentionLogger_priv):
-
-				return
-
-			if not type(message) is unicode:
-				if not type(message) is str:
+			if not isinstance(message, unicode):
+				if not isinstance(message, str):
 					message = unicode(message)
 				else:
 					message = unicode(message, 'utf-8', 'replace')
 
-			levelname = u''
 			componentname = self.__componentName
-			color = COLOR_NORMAL
-			filename = u''
-			linenumber = u''
 			datetime = unicode(time.strftime(u"%b %d %H:%M:%S", time.localtime()), 'utf-8', 'replace')
 			threadId = unicode(thread.get_ident())
 			specialConfig = None
@@ -544,6 +593,9 @@ class LoggerImplementation:
 			elif level == LOG_ESSENTIAL:
 				levelname = u'essential'
 				color = COMMENT_COLOR
+			else:
+				levelname = u''
+				color = COLOR_NORMAL
 
 			try:
 				filename = unicode(os.path.basename(sys._getframe(2).f_code.co_filename))
@@ -563,7 +615,15 @@ class LoggerImplementation:
 						f = f.f_back
 			except Exception:
 				# call stack not deep enough?
-				pass
+				try:
+					filename
+				except NameError:
+					filename = u''
+
+				try:
+					linenumber
+				except NameError:
+					linenumber = u''
 
 			if specialConfig:
 				componentname = specialConfig.get('componentName', componentname)
@@ -572,16 +632,8 @@ class LoggerImplementation:
 				m = self.__messageSubjectFormat
 				if specialConfig:
 					m = specialConfig.get('messageSubjectFormat', m)
-				m = unicode(m)
-				m = m.replace(u'%D', datetime)
-				m = m.replace(u'%T', threadId)
-				m = m.replace(u'%l', unicode(level))
-				m = m.replace(u'%L', levelname)
-				m = m.replace(u'%C', componentname)
-				m = m.replace(u'%M', message)
-				m = m.replace(u'%F', filename)
-				m = m.replace(u'%N', linenumber)
-				if (self.__messageSubjectLevel < LOG_CONFIDENTIAL):
+				m = formatMessage(m)
+				if self.__messageSubjectLevel < LOG_CONFIDENTIAL:
 					for string in self.__confidentialStrings:
 						m = m.replace(string, u'*** confidential ***')
 
@@ -592,28 +644,21 @@ class LoggerImplementation:
 				m = self.__consoleFormat
 				if specialConfig:
 					m = specialConfig.get('consoleFormat', m)
-				m = unicode(m)
-				m = m.replace(u'%D', datetime)
-				m = m.replace(u'%T', threadId)
-				m = m.replace(u'%l', unicode(level))
-				m = m.replace(u'%L', levelname)
-				m = m.replace(u'%C', componentname)
-				m = m.replace(u'%M', message)
-				m = m.replace(u'%F', filename)
-				m = m.replace(u'%N', linenumber)
-				if (self.__consoleLevel < LOG_CONFIDENTIAL):
+				m = formatMessage(m)
+				if self.__consoleLevel < LOG_CONFIDENTIAL:
 					for string in self.__confidentialStrings:
 						m = m.replace(string, u'*** confidential ***')
 
-				fh = sys.stderr
 				if self.__consoleStdout:
 					fh = sys.stdout
+				else:
+					fh = sys.stderr
 
-				fhEncoding = None
 				try:
 					fhEncoding = fh.encoding
-				except:
-					pass
+				except Exception:
+					fhEncoding = None
+
 				if fhEncoding is None:
 					fhEncoding = locale.getpreferredencoding()
 
@@ -632,30 +677,20 @@ class LoggerImplementation:
 					m = self.__fileFormat
 					if specialConfig:
 						m = specialConfig.get('fileFormat', m)
-					m = unicode(m)
-					m = m.replace(u'%D', datetime)
-					m = m.replace(u'%T', threadId)
-					m = m.replace(u'%l', unicode(level))
-					m = m.replace(u'%L', levelname)
-					m = m.replace(u'%C', componentname)
-					m = m.replace(u'%M', message)
-					m = m.replace(u'%F', filename)
-					m = m.replace(u'%N', linenumber)
-					if (self.__fileLevel < LOG_CONFIDENTIAL):
+					m = formatMessage(m)
+					if self.__fileLevel < LOG_CONFIDENTIAL:
 						for string in self.__confidentialStrings:
 							m = m.replace(string, u'*** confidential ***')
 
-					# Open the file
-					lf = None
 					try:
 						lf = codecs.open(logFile, 'a+', 'utf-8', 'replace')
 					except Exception as err:
-						pass
+						lf = None
 
 					if lf:
 						timeout = 0
 						locked = False
-						while (not locked and timeout < 2000):
+						while not locked and timeout < 2000:
 							# While not timed out and not locked
 							try:
 								# Try to lock file
@@ -684,45 +719,20 @@ class LoggerImplementation:
 							lf.write(m)
 							lf.close()
 
-			if level <= self.__syslogLevel:
+			if syslog is not None and level <= self.__syslogLevel:
 				# Log to syslog
 				m = self.__syslogFormat
 				if specialConfig:
 					m = specialConfig.get('syslogFormat', m)
-				m = unicode(m)
-				m = m.replace(u'%D', datetime)
-				m = m.replace(u'%T', threadId)
-				m = m.replace(u'%l', unicode(level))
-				m = m.replace(u'%L', levelname)
-				m = m.replace(u'%C', componentname)
-				m = m.replace(u'%M', message)
-				m = m.replace(u'%F', filename)
-				m = m.replace(u'%N', linenumber)
+				m = formatMessage(m)
 				if self.__syslogLevel < LOG_CONFIDENTIAL:
 					for string in self.__confidentialStrings:
 						m = m.replace(string, u'*** confidential ***')
 
-				if os.name == 'posix':
-					if level == LOG_CONFIDENTIAL:
-						syslog.syslog(syslog.LOG_DEBUG, m)
-					elif level == LOG_DEBUG2:
-						syslog.syslog(syslog.LOG_DEBUG, m)
-					elif level == LOG_DEBUG:
-						syslog.syslog(syslog.LOG_DEBUG, m)
-					elif level == LOG_INFO:
-						syslog.syslog(syslog.LOG_INFO, m)
-					elif level == LOG_NOTICE:
-						syslog.syslog(syslog.LOG_NOTICE, m)
-					elif level == LOG_WARNING:
-						syslog.syslog(syslog.LOG_WARNING, m)
-					elif level == LOG_ERROR:
-						syslog.syslog(syslog.LOG_ERR, m)
-					elif level == LOG_CRITICAL:
-						syslog.syslog(syslog.LOG_CRIT, m)
-					elif level == LOG_COMMENT:
-						syslog.syslog(syslog.LOG_CRIT, m)
-				else:
-					# TODO: not yet implemented
+				try:
+					syslog.syslog(_SYSLOG_LEVEL_MAPPING[level], m)
+				except KeyError:
+					# No known log level - ignoring.
 					pass
 
 			if self.univentionLogger_priv:
@@ -730,36 +740,23 @@ class LoggerImplementation:
 				m = self.__univentionFormat
 				if specialConfig:
 					m = specialConfig.get('univentionFormat', m)
-				m = unicode(m)
-				m = m.replace(u'%D', datetime)
-				m = m.replace(u'%T', threadId)
-				m = m.replace(u'%l', unicode(level))
-				m = m.replace(u'%L', levelname)
-				m = m.replace(u'%C', componentname)
-				m = m.replace(u'%M', message)
-				m = m.replace(u'%F', filename)
-				m = m.replace(u'%N', linenumber)
+				m = formatMessage(m)
 				for string in self.__confidentialStrings:
 					m = m.replace(string, u'*** confidential ***')
 
-				if level == LOG_CONFIDENTIAL:
-					pass
-				elif level == LOG_DEBUG2:
-					self.univentionLogger_priv.debug(self.__univentionClass, self.univentionLogger_priv.ALL, m)
-				elif level == LOG_DEBUG:
-					self.univentionLogger_priv.debug(self.__univentionClass, self.univentionLogger_priv.ALL, m)
-				elif level == LOG_INFO:
-					self.univentionLogger_priv.debug(self.__univentionClass, self.univentionLogger_priv.ALL, m)
+				if level in (LOG_DEBUG2, LOG_DEBUG, LOG_INFO):
+					univentionLevel = self.univentionLogger_priv.ALL
 				elif level == LOG_NOTICE:
-					self.univentionLogger_priv.debug(self.__univentionClass, self.univentionLogger_priv.INFO, m)
+					univentionLevel = self.univentionLogger_priv.INFO
 				elif level == LOG_WARNING:
-					self.univentionLogger_priv.debug(self.__univentionClass, self.univentionLogger_priv.WARN, m)
-				elif level == LOG_ERROR:
-					self.univentionLogger_priv.debug(self.__univentionClass, self.univentionLogger_priv.ERROR, m)
-				elif level == LOG_CRITICAL:
-					self.univentionLogger_priv.debug(self.__univentionClass, self.univentionLogger_priv.ERROR, m)
-				elif level == LOG_COMMENT:
-					self.univentionLogger_priv.debug(self.__univentionClass, self.univentionLogger_priv.ERROR, m)
+					univentionLevel = self.univentionLogger_priv.WARN
+				elif level in (LOG_ERROR, LOG_CRITICAL, LOG_COMMENT):
+					univentionLevel = self.univentionLogger_priv.ERROR
+				else:
+					univentionLevel = None
+
+				if univentionLevel:
+					self.univentionLogger_priv.debug(self.__univentionClass, univentionLevel, m)
 		except Exception as err:
 			if raiseException:
 				raise err
@@ -795,11 +792,8 @@ class LoggerImplementation:
 		the opsi logger.
 		"""
 		def _logWarning(message, category, filename, lineno, file=None, line=None):
-			if file is not None:
-				_showwarning(message, category, filename, lineno, file, line)
-			else:
-				msg = warnings.formatwarning(message, category, filename, lineno)
-				self.warning(msg)
+			formattedMessage = warnings.formatwarning(message, category, filename, lineno, line)
+			self.warning(formattedMessage)
 
 		warnings.showwarning = _logWarning
 
@@ -836,7 +830,7 @@ class LoggerImplementation:
 		''' Log a notice message. '''
 		self.log(LOG_NOTICE, message)
 
-	def warning(self, message,):
+	def warning(self, message):
 		''' Log a warning message. '''
 		self.log(LOG_WARNING, message)
 
@@ -890,7 +884,7 @@ file or syslog.
 		return setattr(self.__instance, attr, value)
 
 
-class VirtFile:
+class VirtFile(object):
 
 	def __init__(self, logger, level):
 		self.logger = logger
