@@ -4,7 +4,7 @@
 # This module is part of the desktop management solution opsi
 # (open pc server integration) - http://www.opsi.org
 
-# Copyright (C) 2006-2015 uib GmbH <info@uib.de>
+# Copyright (C) 2006-2016 uib GmbH <info@uib.de>
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -29,8 +29,6 @@ As an example this contains classes for hosts, products, configurations.
 :license: GNU Affero General Public License version 3
 """
 
-__version__ = '4.0.6.1'
-
 import inspect
 
 from OPSI.Logger import Logger
@@ -50,43 +48,45 @@ from OPSI.Types import (forceActionProgress, forceActionRequest,
 	forceUnicodeList, forceUnicodeLower, forceUnsignedInt, forceUrl)
 from OPSI.Util import fromJson, toJson, generateOpsiHostKey, timestamp
 
+__version__ = '4.0.6.44'
+
 logger = Logger()
+_MANDATORY_CONSTRUCTOR_ARGS_CACHE = {}
 
 
-mandatoryConstructorArgsCache = {}
 def mandatoryConstructorArgs(Class):
 	cacheKey = Class.__name__
 	try:
-		return mandatoryConstructorArgsCache[cacheKey]
+		return _MANDATORY_CONSTRUCTOR_ARGS_CACHE[cacheKey]
 	except KeyError:
-		(args, varargs, varkwargs, defaults) = inspect.getargspec(Class.__init__)
-		if not defaults:
-			defaults = []
-		last = -1 * len(defaults)
-		if last == 0:
-			last = len(args)
-		mandatory = args[1:][:last]
-		logger.debug2(u"mandatoryConstructorArgs for %s: %s" % (Class.__class__, mandatory))
-		mandatoryConstructorArgsCache[cacheKey] = mandatory
+		args, _, _, defaults = inspect.getargspec(Class.__init__)
+		try:
+			last = len(defaults) * -1
+			mandatory = args[1:][:last]
+		except TypeError:  # Happens if defaults is None
+			mandatory = args[1:]
+
+		logger.debug2(u"mandatoryConstructorArgs for {0!r}: {1}".format(Class.__class__, mandatory))
+		_MANDATORY_CONSTRUCTOR_ARGS_CACHE[cacheKey] = mandatory
 		return mandatory
 
 
-def getIdentAttributes(Class):
-	return tuple(mandatoryConstructorArgs(Class))
+def getIdentAttributes(klass):
+	return tuple(mandatoryConstructorArgs(klass))
 
 
-def getForeignIdAttributes(Class):
-	return Class.foreignIdAttributes
+def getForeignIdAttributes(klass):
+	return klass.foreignIdAttributes
 
 
-def getPossibleClassAttributes(Class):
+def getPossibleClassAttributes(klass):
 	"""
 	Returns the possible attributes of a class.
 
 	:returntype: set of strings
 	"""
-	attributes = inspect.getargspec(Class.__init__)[0]
-	for subClass in Class.subClasses.values():
+	attributes = inspect.getargspec(klass.__init__)[0]
+	for subClass in klass.subClasses.values():
 		attributes.extend(inspect.getargspec(subClass.__init__)[0])
 
 	attributes = set(attributes)
@@ -100,24 +100,24 @@ def getPossibleClassAttributes(Class):
 	return attributes
 
 
-def getBackendMethodPrefix(Class):
-	return Class.backendMethodPrefix
+def getBackendMethodPrefix(klass):
+	return klass.backendMethodPrefix
 
 
-def decodeIdent(Class, hash):
+def decodeIdent(klass, hash):
 	try:
 		if hash['ident']:
 			if isinstance(hash['ident'], dict):
 				ident = hash['ident']
 			else:
 				if isinstance(hash['ident'], (str, unicode)):
-					identValues = hash['ident'].split(Class.identSeparator)
+					identValues = hash['ident'].split(klass.identSeparator)
 				elif isinstance(hash['ident'], (tuple, list)):
 					identValues = hash['ident']
 				else:
 					identValues = []
 
-				args = mandatoryConstructorArgs(Class)
+				args = mandatoryConstructorArgs(klass)
 				if len(identValues) == len(args):
 					ident = dict(zip(args, identValues))
 
@@ -146,7 +146,7 @@ def objectsDiffer(obj1, obj2, excludeAttributes=None):
 
 		value2 = obj2.get(attribute)
 
-		if not type(value1) is type(value2):
+		if type(value1) is not type(value2):
 			return True
 
 		if isinstance(value1, dict):
@@ -273,7 +273,15 @@ class BaseObject(object):
 		return not self.__eq__(other)
 
 	def __unicode__(self):
-		return u"<{0}>".format(self.getType())
+		additionalAttributes = []
+		for attr in self.getIdentAttributes():
+			try:
+				value = getattr(self, attr)
+				additionalAttributes.append('{0}={1!r}'.format(attr, value))
+			except AttributeError:
+				pass
+
+		return u"<{0}({1})>".format(self.getType(), ', '.join(additionalAttributes))
 
 	def __str__(self):
 		return self.__unicode__().encode("ascii", "replace")
@@ -302,7 +310,19 @@ class Entity(BaseObject):
 			if varname in hash:
 				kwargs[varname] = hash[varname]
 
-		return Class(**kwargs)
+		try:
+			return Class(**kwargs)
+		except TypeError as error:
+			if '__init__() takes at least' in forceUnicode(error):
+				try:
+					args = mandatoryConstructorArgs(Class)
+					missingArgs = [arg for arg in args if arg not in kwargs]
+					if missingArgs:
+						raise TypeError("Missing required argument(s): {0}".format(', '.join(repr(a) for a in missingArgs)))
+				except NameError:
+					pass
+
+			raise error
 
 	def clone(self, identOnly=False):
 		hash = {}
@@ -350,7 +370,19 @@ class Relationship(BaseObject):
 			if varname in hash:
 				kwargs[varname] = hash[varname]
 
-		return Class(**kwargs)
+		try:
+			return Class(**kwargs)
+		except TypeError as error:
+			if '__init__() takes at least' in forceUnicode(error):
+				try:
+					args = mandatoryConstructorArgs(Class)
+					missingArgs = [arg for arg in args if arg not in kwargs]
+					if missingArgs:
+						raise TypeError("Missing required argument(s): {0}".format(', '.join(repr(a) for a in missingArgs)))
+				except NameError:
+					pass
+
+			raise error
 
 	def clone(self, identOnly=False):
 		hash = {}
@@ -428,9 +460,6 @@ class Object(Entity):
 	def fromJson(jsonString):
 		return fromJson(jsonString, 'Object')
 
-	def __unicode__(self):
-		return u"<{0} id='{1}'>".format(self.getType(), self.id)
-
 Entity.subClasses['Object'] = Object
 
 
@@ -493,9 +522,6 @@ class Host(Object):
 	def fromJson(jsonString):
 		return fromJson(jsonString, 'Host')
 
-	def __unicode__(self):
-		return u"<%s id '%s'>" % (self.getType(), self.id)
-
 Object.subClasses['Host'] = Host
 
 
@@ -569,9 +595,6 @@ class OpsiClient(Host):
 	def fromJson(jsonString):
 		return fromJson(jsonString, 'OpsiClient')
 
-	def __unicode__(self):
-		return u"<%s id '%s'>" % (self.getType(), self.id)
-
 Host.subClasses['OpsiClient'] = OpsiClient
 
 
@@ -700,7 +723,13 @@ class OpsiDepotserver(Host):
 		return fromJson(jsonString, 'OpsiDepotserver')
 
 	def __unicode__(self):
-		return u"<%s id '%s'>" % (self.getType(), self.id)
+		additionalInfos = [u"id={0!r}".format(self.id)]
+		if self.isMasterDepot:
+			additionalInfos.append(u'isMasterDepot={0!r}'.format(self.isMasterDepot))
+		if self.masterDepotId:
+			additionalInfos.append(u"masterDepotId={0!r}".format(self.masterDepotId))
+
+		return u"<{0}({1})>".format(self.getType(), u', '.join(additionalInfos))
 
 Host.subClasses['OpsiDepotserver'] = OpsiDepotserver
 
@@ -853,7 +882,19 @@ class Config(Entity):
 		return fromJson(jsonString, 'Config')
 
 	def __unicode__(self):
-		return u"<%s id '%s'>" % (self.getType(), self.id)
+		return (
+			u"<{klass}(id={id!r}, description={description!r}, "
+			u"possibleValues={possibleValues!r}, defaultValues={defaults!r}, "
+			u"editable={editable!r}, multiValue={multiValue!r})>".format(
+				klass=self.getType(),
+				id=self.id,
+				description=self.description,
+				possibleValues=self.possibleValues,
+				defaults=self.defaultValues,
+				editable=self.editable,
+				multiValue=self.multiValue
+			)
+		)
 
 Entity.subClasses['Config'] = Config
 
@@ -935,6 +976,17 @@ class BoolConfig(Config):
 	def fromJson(jsonString):
 		return fromJson(jsonString, 'BoolConfig')
 
+	def __unicode__(self):
+		return (
+			u"<{klass}(id={id!r}, description={description!r}, "
+			u"defaultValues={defaults!r})>".format(
+				klass=self.getType(),
+				id=self.id,
+				description=self.description,
+				defaults=self.defaultValues,
+			)
+		)
+
 Config.subClasses['BoolConfig'] = BoolConfig
 
 
@@ -988,7 +1040,7 @@ class ConfigState(Relationship):
 		return fromJson(jsonString, 'ConfigState')
 
 	def __unicode__(self):
-		return u"<%s configId '%s', objectId '%s'>" % (self.getType(), self.configId, self.objectId)
+		return u"<{0}(configId={1!r}, objectId={2!r}, values={3!r})>".format(self.getType(), self.configId, self.objectId, self.values)
 
 Relationship.subClasses['ConfigState'] = ConfigState
 
@@ -1211,7 +1263,13 @@ class Product(Entity):
 		return fromJson(jsonString, 'Product')
 
 	def __unicode__(self):
-		return u"<%s id '%s', name '%s'>" % (self.getType(), self.id, self.name)
+		return (
+			u"<{0}(id={1!r}, name={2!r}, productVersion={3!r}, "
+			u"packageVersion={4!r})>".format(
+				self.getType(), self.id, self.name, self.productVersion,
+				self.packageVersion
+			)
+		)
 
 Entity.subClasses['Product'] = Product
 
@@ -1424,10 +1482,6 @@ class ProductProperty(Entity):
 	def fromJson(jsonString):
 		return fromJson(jsonString, 'ProductProperty')
 
-	def __unicode__(self):
-		return u"<%s productId '%s', productVersion '%s', packageVersion '%s', propertyId '%s'>" \
-			% (self.getType(), self.productId, self.productVersion, self.packageVersion, self.propertyId)
-
 Entity.subClasses['ProductProperty'] = ProductProperty
 
 
@@ -1631,8 +1685,13 @@ class ProductDependency(Relationship):
 		return fromJson(jsonString, 'ProductDependency')
 
 	def __unicode__(self):
-		return u"<%s productId '%s', productVersion '%s', packageVersion '%s', productAction '%s', requiredProductId '%s'>" \
-			% (self.getType(), self.productId, self.productVersion, self.packageVersion, self.productAction, self.requiredProductId)
+		return (u"<{klass}(productId={prodId!r}, productVersion={prodVer!r}, "
+				u"packageVersion={packVer!r}, productAction={prodAct!r}, "
+				u"requiredProductId={reqProdId!r}>".format(
+					klass=self.getType(), prodId=self.productId,
+					prodVer=self.productVersion, packVer=self.packageVersion,
+					prodAct=self.productAction,
+					reqProdId=self.requiredProductId))
 
 Relationship.subClasses['ProductDependency'] = ProductDependency
 
@@ -1706,10 +1765,6 @@ class ProductOnDepot(Relationship):
 	def fromJson(jsonString):
 		return fromJson(jsonString, 'ProductOnDepot')
 
-	def __unicode__(self):
-		return u"<%s productId '%s', depotId '%s'>" \
-			% (self.getType(), self.productId, self.depotId)
-
 Relationship.subClasses['ProductOnDepot'] = ProductOnDepot
 
 
@@ -1858,8 +1913,11 @@ class ProductOnClient(Relationship):
 		return fromJson(jsonString, 'ProductOnClient')
 
 	def __unicode__(self):
-		return u"<%s clientId '%s', productId '%s', installationStatus '%s', actionRequest '%s'>" \
-			% (self.getType(), self.clientId, self.productId, self.installationStatus, self.actionRequest)
+		return (u"<{klass}(clientId={clientId!r}, productId={prodId!r}, "
+				u"installationStatus={status!r}, actionRequest={actReq!r})>".format(
+					klass=self.getType(), clientId=self.clientId,
+					prodId=self.productId, status=self.installationStatus,
+					actReq=self.actionRequest))
 
 Relationship.subClasses['ProductOnClient'] = ProductOnClient
 
@@ -1920,10 +1978,6 @@ class ProductPropertyState(Relationship):
 	def fromJson(jsonString):
 		return fromJson(jsonString, 'ProductPropertyState')
 
-	def __unicode__(self):
-		return u"<%s productId '%s', propertyId '%s', objectId '%s'>" \
-			% (self.getType(), self.productId, self.propertyId, self.objectId)
-
 Relationship.subClasses['ProductPropertyState'] = ProductPropertyState
 
 
@@ -1969,7 +2023,8 @@ class Group(Object):
 		return fromJson(jsonString, 'Group')
 
 	def __unicode__(self):
-		return u"<%s id '%s', parentGroupId '%s'>" % (self.getType(), self.id, self.parentGroupId)
+		return (u"<{klass}(id={id!r} parentGroupId={parentId!r}>".format(
+				klass=self.getType(), id=self.id, parentId=self.parentGroupId))
 
 Object.subClasses['Group'] = Group
 
@@ -2067,10 +2122,6 @@ class ObjectToGroup(Relationship):
 	def fromJson(jsonString):
 		return fromJson(jsonString, 'ObjectToGroup')
 
-	def __unicode__(self):
-		return u"<%s groupType '%s', groupId '%s', objectId '%s'>" \
-			% (self.getType(), self.groupType, self.groupId, self.objectId)
-
 Relationship.subClasses['ObjectToGroup'] = ObjectToGroup
 
 
@@ -2174,8 +2225,20 @@ class LicenseContract(Entity):
 		return fromJson(jsonString, 'LicenseContract')
 
 	def __unicode__(self):
-		return u"<%s id '%s', description '%s'>" \
-			% (self.getType(), self.id, self.description)
+		infos = [u"id={0!r}".format(self.id)]
+
+		if self.description:
+			infos.append(u"description={0!r}".format(self.description))
+		if self.partner:
+			infos.append(u"partner={0!r}".format(self.partner))
+		if self.conclusionDate:
+			infos.append(u"conclusionDate={0!r}".format(self.conclusionDate))
+		if self.notificationDate:
+			infos.append(u"notificationDate={0!r}".format(self.notificationDate))
+		if self.expirationDate:
+			infos.append(u"expirationDate={0!r}".format(self.expirationDate))
+
+		return u"<{0}({1})>".format(self.getType(), u', '.join(infos))
 
 Entity.subClasses['LicenseContract'] = LicenseContract
 
@@ -2251,8 +2314,18 @@ class SoftwareLicense(Entity):
 		return fromJson(jsonString, 'SoftwareLicense')
 
 	def __unicode__(self):
-		return u"<%s id '%s', licenseContractId '%s'>" \
-			% (self.getType(), self.id, self.licenseContractId)
+		infos = [
+			u"id='{0}'".format(self.id),
+			u"licenseContractId='{0}'".format(self.licenseContractId)
+		]
+		if self.maxInstallations:
+			infos.append(u'maxInstallations={0}'.format(self.maxInstallations))
+		if self.boundToHost:
+			infos.append(u"boundToHost={0!r}".format(self.boundToHost))
+		if self.expirationDate:
+			infos.append(u"expirationDate={0!r}".format(self.expirationDate))
+
+		return u"<{0}({1})>".format(self.getType(), u', '.join(infos))
 
 Entity.subClasses['LicenseContract'] = LicenseContract
 
@@ -2305,7 +2378,7 @@ class OEMSoftwareLicense(SoftwareLicense):
 	def setBoundToHost(self, boundToHost):
 		self.boundToHost = forceHostId(boundToHost)
 		if not self.boundToHost:
-			raise BackendBadValueError("OEM software license requires boundToHost value")
+			raise BackendBadValueError(u"OEM software license requires boundToHost value")
 
 	@staticmethod
 	def fromHash(hash):
@@ -2434,8 +2507,14 @@ class LicensePool(Entity):
 		return fromJson(jsonString, 'LicensePool')
 
 	def __unicode__(self):
-		return u"<%s id '%s', description '%s'>" \
-			% (self.getType(), self.id, self.description)
+		infos = [u"id={0!r}".format(self.id)]
+
+		if self.description:
+			infos.append(u"description={0!r}".format(self.description))
+		if self.productIds:
+			infos.append(u"productIds={0!r}".format(self.productIds))
+
+		return u"<{0}({1})>".format(self.getType(), u', '.join(infos))
 
 Entity.subClasses['LicensePool'] = LicensePool
 
@@ -2515,8 +2594,21 @@ class AuditSoftwareToLicensePool(Relationship):
 		return fromJson(jsonString, 'AuditSoftwareToLicensePool')
 
 	def __unicode__(self):
-		return u"<%s licensePoolId '%s', name '%s'>" \
-			% (self.getType(), self.licensePoolId, self.name)
+		infos = [u"name={0}".format(self.name)]
+
+		if self.version:
+			infos.append(u"version={0!r}".format(self.version))
+		if self.subVersion:
+			infos.append(u"subVersion={0!r}".format(self.subVersion))
+		if self.language:
+			infos.append(u"language={0!r}".format(self.language))
+		if self.architecture:
+			infos.append(u"architecture={0!r}".format(self.architecture))
+		if self.licensePoolId:
+			infos.append(u"licensePoolId={0!r}".format(self.licensePoolId))
+
+		return u"<{0}({1})>".format(self.getType(), u', '.join(infos))
+
 
 Relationship.subClasses['AuditSoftwareToLicensePool'] = AuditSoftwareToLicensePool
 
@@ -2570,10 +2662,6 @@ class SoftwareLicenseToLicensePool(Relationship):
 	def fromJson(jsonString):
 		return fromJson(jsonString, 'SoftwareLicenseToLicensePool')
 
-	def __unicode__(self):
-		return u"<%s softwareLicenseId '%s', licensePoolId '%s'>" \
-			% (self.getType(), self.softwareLicenseId, self.licensePoolId)
-
 Relationship.subClasses['SoftwareLicenseToLicensePool'] = SoftwareLicenseToLicensePool
 
 
@@ -2645,10 +2733,6 @@ class LicenseOnClient(Relationship):
 	def fromJson(jsonString):
 		return fromJson(jsonString, 'LicenseOnClient')
 
-	def __unicode__(self):
-		return u"<%s softwareLicenseId '%s', licensePoolId '%s', clientId '%s'>" \
-			% (self.getType(), self.softwareLicenseId, self.licensePoolId, self.clientId)
-
 Relationship.subClasses['LicenseOnClient'] = LicenseOnClient
 
 
@@ -2757,10 +2841,6 @@ class AuditSoftware(Entity):
 	def fromJson(jsonString):
 		return fromJson(jsonString, 'AuditSoftware')
 
-	def __unicode__(self):
-		return u"<%s name '%s', version '%s', subVersion '%s', language '%s', architecture '%s'>" \
-			% (self.getType(), self.name, self.version, self.subVersion, self.language, self.architecture)
-
 Entity.subClasses['AuditSoftware'] = AuditSoftware
 
 
@@ -2925,10 +3005,6 @@ class AuditSoftwareOnClient(Relationship):
 	def fromJson(jsonString):
 		return fromJson(jsonString, 'AuditSoftwareOnClient')
 
-	def __unicode__(self):
-		return u"<%s name '%s', version '%s', subVersion '%s', language '%s', architecture '%s', clientId '%s'>" \
-			% (self.getType(), self.name, self.version, self.subVersion, self.language, self.architecture, self.clientId)
-
 Relationship.subClasses['AuditSoftwareOnClient'] = AuditSoftwareOnClient
 
 
@@ -2983,19 +3059,34 @@ class AuditHardware(Entity):
 					raise BackendConfigurationError(u"Attribute '%s' of hardware class '%s' has unknown type '%s'" % (attribute, hardwareClass, type))
 		else:
 			for (attribute, value) in kwargs.items():
-				if type(value) is str:
+				if isinstance(value, str):
 					kwargs[attribute] = forceUnicode(value).strip()
 
 		self.__dict__.update(kwargs)
 
-		if hasattr(self, 'vendorId') and self.vendorId:
-			self.vendorId = forceHardwareVendorId(self.vendorId)
-		if hasattr(self, 'subsystemVendorId') and self.subsystemVendorId:
-			self.subsystemVendorId = forceHardwareVendorId(self.subsystemVendorId)
-		if hasattr(self, 'deviceId') and self.deviceId:
-			self.deviceId = forceHardwareDeviceId(self.deviceId)
-		if hasattr(self, 'subsystemDeviceId') and self.subsystemDeviceId:
-			self.subsystemDeviceId = forceHardwareDeviceId(self.subsystemDeviceId)
+		try:
+			if self.vendorId:
+				self.vendorId = forceHardwareVendorId(self.vendorId)
+		except AttributeError:
+			pass
+
+		try:
+			if self.subsystemVendorId:
+				self.subsystemVendorId = forceHardwareVendorId(self.subsystemVendorId)
+		except AttributeError:
+			pass
+
+		try:
+			if self.deviceId:
+				self.deviceId = forceHardwareDeviceId(self.deviceId)
+		except AttributeError:
+			pass
+
+		try:
+			if self.subsystemDeviceId:
+				self.subsystemDeviceId = forceHardwareDeviceId(self.subsystemDeviceId)
+		except AttributeError:
+			pass
 
 	@staticmethod
 	def setHardwareConfig(hardwareConfig):
@@ -3033,7 +3124,7 @@ class AuditHardware(Entity):
 			if key == 'type':
 				continue
 
-			if type(key) is unicode:
+			if isinstance(key, unicode):
 				key = str(key)
 			initHash[key] = value
 		return AuditHardware(**initHash)
@@ -3043,14 +3134,41 @@ class AuditHardware(Entity):
 		return fromJson(jsonString, 'AuditHardware')
 
 	def __unicode__(self):
-		infos = [self.getType()]
+		infos = []
 		hardwareClass = self.getHardwareClass()
 		if hardwareClass:
-			infos.append(u"hardwareClass '{0}'".format(hardwareClass))
-		if hasattr(self, 'name'):
-			infos.append(u"name '{0}'".format(self.name))
+			infos.append(u"hardwareClass={0!r}".format(hardwareClass))
 
-		return u"<{0}>".format(', '.join(infos))
+		try:
+			infos.append(u"name={0!r}".format(self.name))
+		except AttributeError:
+			pass
+
+		try:
+			if self.vendorId:
+				infos.append(u"vendorId={0!r}".format(self.vendorId))
+		except AttributeError:
+			pass
+
+		try:
+			if self.subsystemVendorId:
+				infos.append(u"subsystemVendorId={0!r}".format(self.subsystemVendorId))
+		except AttributeError:
+			pass
+
+		try:
+			if self.deviceId:
+				infos.append(u"deviceId={0!r}".format(self.deviceId))
+		except AttributeError:
+			pass
+
+		try:
+			if self.subsystemDeviceId:
+				infos.append(u"subsystemDeviceId={0!r}".format(self.subsystemDeviceId))
+		except AttributeError:
+			pass
+
+		return u"<{0}({1})>".format(self.__class__.__name__, u', '.join(infos))
 
 Entity.subClasses['AuditHardware'] = AuditHardware
 
@@ -3067,11 +3185,13 @@ class AuditHardwareOnHost(Relationship):
 		self.state = None
 		self.setHostId(hostId)
 		self.setHardwareClass(hardwareClass)
+
 		for attribute in self.hardwareAttributes.get(hardwareClass, {}):
 			if attribute not in kwargs:
-				if attribute.lower() in kwargs:
-					kwargs[attribute] = kwargs[attribute.lower()]
-					del kwargs[attribute.lower()]
+				lowerAttribute = attribute.lower()
+				if lowerAttribute in kwargs:
+					kwargs[attribute] = kwargs[lowerAttribute]
+					del kwargs[lowerAttribute]
 				else:
 					kwargs[attribute] = None
 
@@ -3110,7 +3230,7 @@ class AuditHardwareOnHost(Relationship):
 					raise BackendConfigurationError(u"Attribute '%s' of hardware class '%s' has unknown type '%s'" % (attribute, hardwareClass, type))
 		else:
 			for (attribute, value) in kwargs.items():
-				if type(value) is str:
+				if isinstance(value, str):
 					kwargs[attribute] = forceUnicode(value).strip()
 
 		self.__dict__.update(kwargs)
@@ -3121,14 +3241,29 @@ class AuditHardwareOnHost(Relationship):
 		if state is not None:
 			self.setState(state)
 
-		if hasattr(self, 'vendorId') and self.vendorId:
-			self.vendorId = forceHardwareVendorId(self.vendorId)
-		if hasattr(self, 'subsystemVendorId') and self.subsystemVendorId:
-			self.subsystemVendorId = forceHardwareVendorId(self.subsystemVendorId)
-		if hasattr(self, 'deviceId') and self.deviceId:
-			self.deviceId = forceHardwareDeviceId(self.deviceId)
-		if hasattr(self, 'subsystemDeviceId') and self.subsystemDeviceId:
-			self.subsystemDeviceId = forceHardwareDeviceId(self.subsystemDeviceId)
+		try:
+			if self.vendorId:
+				self.vendorId = forceHardwareVendorId(self.vendorId)
+		except AttributeError:
+			pass
+
+		try:
+			if self.subsystemVendorId:
+				self.subsystemVendorId = forceHardwareVendorId(self.subsystemVendorId)
+		except AttributeError:
+			pass
+
+		try:
+			if self.deviceId:
+				self.deviceId = forceHardwareDeviceId(self.deviceId)
+		except AttributeError:
+			pass
+
+		try:
+			if self.subsystemDeviceId:
+				self.subsystemDeviceId = forceHardwareDeviceId(self.subsystemDeviceId)
+		except AttributeError:
+			pass
 
 	@staticmethod
 	def setHardwareConfig(hardwareConfig):
@@ -3213,7 +3348,7 @@ class AuditHardwareOnHost(Relationship):
 			if key == 'type':
 				continue
 
-			if type(key) is unicode:
+			if isinstance(key, unicode):
 				key = str(key)
 
 			initHash[key] = value
@@ -3224,14 +3359,17 @@ class AuditHardwareOnHost(Relationship):
 		return fromJson(jsonString, 'AuditHardwareOnHost')
 
 	def __unicode__(self):
-		additional = [u"hostId='{0}'".format(self.hostId)]
+		additional = [u"hostId={0!r}".format(self.hostId)]
 		hardwareClass = self.getHardwareClass()
 		if hardwareClass:
-			additional.append(u"hardwareClass='{0}'".format(hardwareClass))
-		if hasattr(self, 'name'):
-			additional.append(u"name='{0}'".format(forceUnicode(self.name)))
+			additional.append(u"hardwareClass={0!r}".format(hardwareClass))
 
-		return u"<{type} {additional}>".format(
+		try:
+			additional.append(u"name={0!r}".format(forceUnicode(self.name)))
+		except AttributeError:
+			pass
+
+		return u"<{type}({additional})>".format(
 			type=self.getType(),
 			additional=u', '.join(additional)
 		)
@@ -3395,7 +3533,8 @@ class BootConfiguration(Relationship):
 		return fromJson(jsonString, 'BootConfiguration')
 
 	def __unicode__(self):
-		return u"<%s name '%s', clientId '%s', priority %d>" \
-			% (self.getType(), self.name, self.clientId, self.priority)
+		return u"<{klass}(name={name!r}, clientId={cid!r}, priority={prio:d}>".format(
+			klass=self.getType(), name=self.name,
+			cid=self.clientId, prio=self.priority)
 
 Relationship.subClasses['BootConfiguration'] = BootConfiguration
