@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # This file is part of python-opsi.
-# Copyright (C) 2010-2015 uib GmbH <info@uib.de>
+# Copyright (C) 2010-2016 uib GmbH <info@uib.de>
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -27,8 +27,6 @@ Algorithms to get a product order for an installation.
 :license: GNU Affero General Public License version 3
 """
 
-__version__ = '4.0.6'
-
 from collections import defaultdict
 
 from OPSI.Logger import Logger
@@ -36,6 +34,8 @@ from OPSI.Object import ProductOnClient
 from OPSI.Types import OpsiProductOrderingError, BackendUnaccomplishableError
 from OPSI.Types import forceInt, forceBool
 
+__version__ = '4.0.6.41'
+
 logger = Logger()
 
 
@@ -43,8 +43,9 @@ class CircularProductDependencyError(BackendUnaccomplishableError):
 	ExceptionShortDescription = "A circular dependency between products."
 
 
-def addActionRequest(productOnClientByProductId, productId, productDependenciesByProductId, availableProductsByProductId, addedInfo={}):
+def addActionRequest(productOnClientByProductId, productId, productDependenciesByProductId, availableProductsByProductId, addedInfo=None):
 	logger.debug(u"checking dependencies for product '%s', action '%s'" % (productId, productOnClientByProductId[productId].actionRequest))
+	addedInfo = addedInfo or {}
 
 	poc = productOnClientByProductId[productId]
 	if poc.actionRequest == 'none' or not productDependenciesByProductId.get(productId):
@@ -138,8 +139,6 @@ def addActionRequest(productOnClientByProductId, productId, productDependenciesB
 				)
 			)
 			continue
-			#raise BackendUnaccomplishableError(u"Cannot fulfill dependency of product '%s' to product '%s': action '%s' needed but action '%s' already set" \
-			#		% (productId, dependency.requiredProductId, requiredAction, productOnClientsByProductId[dependency.requiredProductId].actionRequest))
 		logger.info(u"   => adding action '%s' for product '%s'" % (requiredAction, dependency.requiredProductId))
 
 		if dependency.requiredProductId in addedInfo:
@@ -197,7 +196,10 @@ def addDependentProductOnClients(productOnClients, availableProducts, productDep
 
 
 class OrderRequirement(object):
-	# Represents a request for ordering of two elements with a notice if it is fulfilled
+	"""
+	Represents a request for ordering of two elements with a notice
+	if it is fulfilled.
+	"""
 
 	def __init__(self, prior, posterior, fulfilled=False):
 		self.prior = forceInt(prior)
@@ -205,7 +207,9 @@ class OrderRequirement(object):
 		self.fulfilled = forceBool(fulfilled)
 
 	def __unicode__(self):
-		return u"<OrderRequirement prior '%s', posterior '%s', fulfilled '%s'>" % (self.prior, self.posterior, self.fulfilled)
+		return (u"<OrderRequirement(prior={prio!r}, posterior={post!r}, "
+				u"fulfilled={ful!r}>".format(
+					prio=self.prior, post=self.posterior, ful=self.fulfilled))
 
 	def __str__(self):
 		return self.__unicode__().encode("ascii", "replace")
@@ -217,7 +221,7 @@ class OrderRequirement(object):
 class Requirements(object):
 	# Comprises a list with ordering requirements and ordered lists of them
 
-	def __init__(self, allItemsCount):
+	def __init__(self):
 		self.list = []
 		self.orderByPrior = []
 		self.orderByPosterior = []
@@ -354,7 +358,7 @@ class Requirements(object):
 
 		errorMessage = u'Potentially conflicting requirements for: {0}'.format(candidatesCausingProblemes)
 		logger.error(errorMessage)
-		raise OpsiProductOrderingError(errorMessage)
+		raise OpsiProductOrderingError(errorMessage, candidatesCausingProblemes)
 
 	def getCount(self):
 		return len(self.list)
@@ -489,7 +493,7 @@ def generateProductOnClientSequence(productOnClients, sortedList):
 		productOnClientsByClientIdAndProductId[productOnClient.clientId][productOnClient.productId] = productOnClient
 
 	productOnClients = []
-	for (clientId, productOnClientsByProductId) in productOnClientsByClientIdAndProductId.items():
+	for productOnClientsByProductId in productOnClientsByClientIdAndProductId.values():
 		sequence = 0
 		for productId in sortedList:
 			if productId in productOnClientsByProductId:
@@ -509,6 +513,8 @@ def generateProductOnClientSequence(productOnClients, sortedList):
 
 
 def generateProductSequence_algorithm1(availableProducts, productDependencies):
+	logger.notice(u"Generating product sequence with algorithm 1.")
+
 	# Build priority classes and indices
 	logger.debug(u"*********running algorithm1")
 	logger.debug(u"availableProducts %s " % availableProducts)
@@ -581,7 +587,7 @@ def generateProductSequence_algorithm1(availableProducts, productDependencies):
 	logger.debug(u"requs %s " % requs)
 
 	try:
-		requObjects = Requirements(len(requirements))
+		requObjects = Requirements()
 		for item in requs:
 			requObj = OrderRequirement(item[0], item[1], False)
 			logger.debug(u"requObj %s " % requObj)
@@ -589,15 +595,19 @@ def generateProductSequence_algorithm1(availableProducts, productDependencies):
 
 		ob = OrderBuild(len(availableProducts), requObjects, False)
 		try:
-			for k in range(len(availableProducts)):
+			for _ in availableProducts:
 				ob.proceed()
-				logger.debug(u"ordering '%s' " % ob.getOrdering())
-
+				logger.debug(u"ordering {0!r}".format(ob.getOrdering()))
 		except OpsiProductOrderingError as error:
 			logger.warning(u"algo1 catched OpsiProductOrderingError: {0}".format(error))
-			for i in range(len(availableProducts)):
-				logger.warning(u" product %s %s " % (i, availableProducts[i].getId()))
-			raise error
+			for i, product in enumerate(availableProducts):
+				logger.warning(u" product {0} {1}".format(i, product.getId()))
+
+			raise OpsiProductOrderingError(
+				u'Potentially conflicting requirements for: {0}'.format(
+					', '.join([availableProducts[index].id for index in error.problematicRequirements])
+				)
+			)
 
 		ordering = ob.getOrdering()
 		logger.debug(u"completed ordering '%s' " % ordering)
@@ -606,7 +616,6 @@ def generateProductSequence_algorithm1(availableProducts, productDependencies):
 			sortedList.append(productIds[idx])
 
 		logger.debug(u"sortedList algo1 '%s' " % sortedList)
-
 	except OpsiProductOrderingError as error:
 		logger.warning(u"algo1 outer catched OpsiProductOrderingError: {0}".format(error))
 		raise error
@@ -619,9 +628,7 @@ def generateProductSequence_algorithm1(availableProducts, productDependencies):
 	prioClassStart = 100
 
 	while shrinkingSortedList:
-
 		prioClassHead = -100
-
 		productHeading = None
 		for productId in shrinkingSortedList:
 			logger.debug(u"product %s " % productId)
@@ -632,7 +639,7 @@ def generateProductSequence_algorithm1(availableProducts, productDependencies):
 
 			logger.debug(u"product %s has priority class %s, prioClassHead now  %s " % (productId, prioClass, prioClassHead))
 
-		#get all products with priority class <= prioClassHead
+		# get all products with priority class <= prioClassHead
 		prioList = range(0, prioClassStart - prioClassHead)
 		for p in prioList:
 			q = prioClassStart - p
@@ -673,12 +680,15 @@ def generateProductSequence_algorithm1(availableProducts, productDependencies):
 
 
 def generateProductOnClientSequence_algorithm1(productOnClients, availableProducts, productDependencies):
+	logger.notice(u"Generating productOnClient sequence with algorithm 1.")
 	sortedProductList = generateProductSequence_algorithm2(availableProducts, productDependencies)
 	productOnClients = generateProductOnClientSequence(productOnClients, sortedProductList)
 	return productOnClients
 
 
 def generateProductSequence_algorithm2(availableProducts, productDependencies):
+	logger.notice(u"Generating product sequence with algorithm 2.")
+
 	# Build priority classes and indices
 	logger.debug(u"*********running algorithm2")
 	logger.debug(u"availableProducts %s " % availableProducts)
@@ -717,28 +727,23 @@ def generateProductSequence_algorithm2(availableProducts, productDependencies):
 	requirementsByClasses = defaultdict(list)
 
 	for (prod1, prod2) in setupRequirements:
-		logger.debug(u"Product 1 %s" % prod1)
+		logger.debug(u"First product: {0}".format(prod1))
 		if prod1 not in productById:
 			logger.warning(u"Product %s is requested but not available" % prod1)
 			continue
 
-		prio1 = productById[prod1].priority
-		if not prio1:
-			prio1 = 0
-
-		logger.debug(u"Product 2 %s" % prod2)
+		logger.debug(u"Second product: {0}".format(prod2))
 		if prod2 not in productById:
 			logger.warning(u"Product %s is requested but not available" % prod2)
 			continue
 
-		prio2 = productById[prod2].priority
-		if not prio2:
-			prio2 = 0
+		prio1 = productById[prod1].priority or 0
+		prio2 = productById[prod2].priority or 0
 
-		logger.debug(u"Priority product 1 %s" % prio1)
-		logger.debug(u"Priority product 2 %s" % prio2)
+		logger.debug(u"Priority {0}: {1}".format(prod1, prio1))
+		logger.debug(u"Priority {0}: {1}".format(prod2, prio2))
 		if prio1 > prio2:
-			logger.notice(u"The ordering is guaranteed by priority handling")
+			logger.debug(u"The ordering is guaranteed by priority handling")
 		elif prio1 < prio2:
 			logger.warning(u"Dependency declaration between %s and %s contradicts priority declaration, will be ignored" % (prod1, prod2))
 		else:
@@ -760,20 +765,24 @@ def generateProductSequence_algorithm2(availableProducts, productDependencies):
 
 			if prioclasskey in requirementsByClasses:
 				requs = requirementsByClasses[prioclasskey]
-				requObjects = Requirements(len(prioclass))
+				requObjects = Requirements()
 				for item in requs:
 					requObjects.add(OrderRequirement(item[0], item[1], False))
 
 				ob = OrderBuild(len(prioclass), requObjects, True)
 				try:
-					for k in range(len(prioclass)):
+					for _ in prioclass:
 						ob.proceed()
-
 				except OpsiProductOrderingError as error:
 					logger.warning(u"algo2 catched OpsiProductOrderingError: {0}".format(error))
-					for i in range(len(prioclass)):
-						logger.warning(u" product %s %s " % (i, prioclass[i]))
-					raise error
+					for i, prio in enumerate(prioclass):
+						logger.warning(u" product {0} {1}".format(i, prio))
+
+					raise OpsiProductOrderingError(
+						u"Potentially conflicting requirements for: {0}".format(
+							', '.join([prioclass[index] for index in error.problematicRequirements])
+						)
+					)
 
 				orderingsByClasses[prioclasskey] = ob.getOrdering()
 				logger.debug(u"prioclasskey, ordering '%s' , '%s'" % (prioclasskey, ob.getOrdering()))
@@ -793,7 +802,6 @@ def generateProductSequence_algorithm2(availableProducts, productDependencies):
 					sortedList.append(element)
 
 		logger.debug(u"sortedList algo2  '%s' " % sortedList)
-
 	except OpsiProductOrderingError as error:
 		logger.warning(u"algo2 outer catched OpsiProductOrderingError: {0}".format(error))
 		raise error
@@ -802,6 +810,7 @@ def generateProductSequence_algorithm2(availableProducts, productDependencies):
 
 
 def generateProductOnClientSequence_algorithm2(productOnClients, availableProducts, productDependencies):
+	logger.notice(u"Generating productOnClient sequence with algorithm 2.")
 	sortedProductList = generateProductSequence_algorithm2(availableProducts, productDependencies)
 	productOnClients = generateProductOnClientSequence(productOnClients, sortedProductList)
 	return productOnClients
@@ -813,6 +822,8 @@ def _generateProductOnClientSequence_algorithm3(productOnClients, availableProdu
 
 	This is the *old* variant and only available for historic reasons.
 	"""
+	logger.notice(u"Generating productOnClient sequence with algorithm 3.")
+
 	logger.debug(u"*********  running algorithm3")
 	productDependenciesByProductId = defaultdict(list)
 	for productDependency in productDependencies:
@@ -838,8 +849,8 @@ def _generateProductOnClientSequence_algorithm3(productOnClients, availableProdu
 		productSequence.extend(priorityToProductIds[priority])
 
 	logger.debug2(u"Sequence of available products after priority sorting:")
-	for i in range(len(productSequence)):
-		logger.debug2(u"   [%2.0f] %s" % (i, productSequence[i]))
+	for i, product in enumerate(productSequence):
+		logger.debug2(u"   [{0}] {1}".format(i, product))
 
 	sortedProductOnClients = []
 
@@ -889,9 +900,9 @@ def _generateProductOnClientSequence_algorithm3(productOnClients, availableProdu
 						logger.debug("%s requires %s %s => no sequence change required." % (productId, requiredProductId, requirementType))
 
 		logger.debug2(u"Sequence of available products after dependency sorting (client %s):" % clientId)
-		for i in range(len(sequence)):
-			logger.debug2(u"   [%2.0f] %s" % (i, sequence[i]))
-			productOnClient = productOnClientByProductId[sequence[i]]
+		for i, productId in enumerate(sequence):
+			logger.debug2(u"   [{0}] {1}".format(i, productId))
+			productOnClient = productOnClientByProductId[productId]
 			productOnClient.setActionSequence(i + 1)
 			sortedProductOnClients.append(productOnClient)
 
