@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # This file is part of python-opsi.
-# Copyright (C) 2006-2014 uib GmbH <info@uib.de>
+# Copyright (C) 2006-2016 uib GmbH <info@uib.de>
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -30,8 +30,6 @@ It also is home to various exception classes.
 :license: GNU Affero General Public License version 3
 """
 
-__version__ = '4.0.6.1'
-
 import datetime
 import os
 import re
@@ -41,6 +39,8 @@ import types
 
 from OPSI.Logger import Logger
 
+__version__ = '4.0.6.44'
+
 encoding = sys.getfilesystemencoding()
 logger = Logger()
 
@@ -71,6 +71,18 @@ _LICENSE_POOL_ID_REGEX = re.compile('^[a-z0-9][a-z0-9-_\. :]*$')
 _LANGUAGE_CODE_REGEX = re.compile('^([a-z]{2,3})[-_]?([a-z]{4})?[-_]?([a-z]{2})?$')
 _ARCHITECTURE_REGEX = re.compile('^(x86|x64)$')
 
+if sys.version_info > (3, ):
+	# Python 3
+	unicode = str
+	_STRING_TYPE = str
+	_UNICODE_TYPE = str
+	_STRING_TYPES = (str, )
+else:
+	# Python 2
+	_STRING_TYPE = str
+	_UNICODE_TYPE = unicode
+	_STRING_TYPES = (str, unicode)
+
 
 def forceList(var):
 	if not isinstance(var, (set, list, tuple, types.GeneratorType)):
@@ -80,11 +92,11 @@ def forceList(var):
 
 
 def forceUnicode(var):
-	if type(var) is types.UnicodeType:
+	if isinstance(var, _UNICODE_TYPE):
 		return var
-	elif type(var) is types.StringType:
+	elif isinstance(var, _STRING_TYPE):
 		return unicode(var, 'utf-8', 'replace')
-	elif (os.name == 'nt') and type(var) is WindowsError:
+	elif (os.name == 'nt') and isinstance(var, WindowsError):
 		return u"[Error %s] %s" % (var.args[0], var.args[1].decode(encoding))
 
 	if hasattr(var, '__unicode__'):
@@ -98,11 +110,13 @@ def forceUnicode(var):
 	except Exception:
 		pass
 
-	if hasattr(var, '__repr__'):
+	try:
 		var = var.__repr__()
-		if type(var) is types.UnicodeType:
+		if isinstance(var, _UNICODE_TYPE):
 			return var
 		return unicode(var, 'utf-8', 'replace')
+	except Exception:
+		pass
 
 	return unicode(var)
 
@@ -128,14 +142,15 @@ def forceUnicodeLowerList(var):
 
 
 def forceBool(var):
-	if type(var) is types.BooleanType:
+	if isinstance(var, bool):
 		return var
-
-	if type(var) in (types.UnicodeType, types.StringType):
-		if var.lower() in ('true', 'yes', 'on', '1'):
-			return True
-		elif var.lower() in ('false', 'no', 'off', '0'):
-			return False
+	elif isinstance(var, _STRING_TYPES):
+		if len(var) <= 5:  # longest word is 5 characters ("false")
+			lowValue = var.lower()
+			if lowValue in ('true', 'yes', 'on', '1'):
+				return True
+			elif lowValue in ('false', 'no', 'off', '0'):
+				return False
 
 	return bool(var)
 
@@ -145,7 +160,7 @@ def forceBoolList(var):
 
 
 def forceInt(var):
-	if type(var) is types.IntType:
+	if isinstance(var, int):
 		return var
 	try:
 		return int(var)
@@ -165,28 +180,29 @@ def forceUnsignedInt(var):
 
 
 def forceOct(var):
-	if type(var) is types.IntType:
+	if isinstance(var, int):
 		return var
 
 	try:
 		tmp = forceUnicode(var)
 		var = ''
-		for i in xrange(len(tmp)):
-			x = forceInt(tmp[i])
+		for i, x in enumerate(tmp):
+			x = forceInt(x)
 			if x > 7:
-				raise Exception('"{0}" is too big')
+				raise ValueError(u'{0!r} is too big'.format(x))
 			elif i == 0 and x != '0':
 				var += '0'
 			var += str(x)
 		var = eval(var)
 		return var
-	except Exception as e:
-		raise ValueError(u"Bad oct value '%s': %s" % (var, e))
+	except Exception as error:
+		raise ValueError(u"Bad oct value {0!r}: {1}".format(var, error))
 
 
 def forceFloat(var):
-	if type(var) is types.FloatType:
+	if isinstance(var, float):
 		return var
+
 	try:
 		return float(var)
 	except Exception as e:
@@ -196,18 +212,27 @@ def forceFloat(var):
 def forceDict(var):
 	if var is None:
 		return {}
-	elif type(var) is types.DictType:
+	elif isinstance(var, dict):
 		return var
 
 	raise ValueError(u"Not a dict '%s'" % var)
 
 
 def forceTime(var):
-	if type(var) is time.struct_time:
+	"""
+	Convert `var` to a time.struct_time.
+
+	If no conversion is possible a `ValueError` will be raised.
+	"""
+	if isinstance(var, time.struct_time):
 		return var
-	if type(var) in (types.IntType, types.FloatType):
+	elif isinstance(var, datetime.datetime):
+		var = time.mktime(var.timetuple()) + var.microsecond / 1E6
+
+	if isinstance(var, (int, float)):
 		return time.localtime(var)
-	raise ValueError(u"Not a time '%s'" % var)
+
+	raise ValueError(u"Not a time {0!r}".format(var))
 
 
 def forceHardwareVendorId(var):
@@ -225,16 +250,25 @@ def forceHardwareDeviceId(var):
 
 
 def forceOpsiTimestamp(var):
+	"""
+	Make `var` an opsi-compatible timestamp.
+
+	This is a string with the format "YYYY-MM-DD HH:MM:SS".
+
+	If a conversion is not possible a `ValueError` will be raised.
+	"""
 	if not var:
-		var = u'0000-00-00 00:00:00'
-	if isinstance(var, datetime.datetime):
-		var = str(var)
+		return u'0000-00-00 00:00:00'
+	elif isinstance(var, datetime.datetime):
+		return forceUnicode(var.strftime('%Y-%m-%d %H:%M:%S'))
+
 	var = forceUnicode(var)
 	match = re.search(_OPSI_TIMESTAMP_REGEX, var)
 	if not match:
 		match = re.search(_OPSI_DATE_REGEX, var)
 		if not match:
-			raise ValueError(u"Bad opsi timestamp: '%s'" % var)
+			raise ValueError(u"Bad opsi timestamp: {0!r}".format(var))
+
 		return u'%s-%s-%s 00:00:00' % (match.group(1), match.group(2), match.group(3))
 
 	return u'%s-%s-%s %s:%s:%s' % (match.group(1), match.group(2), match.group(3), match.group(4), match.group(5), match.group(6))
@@ -363,10 +397,10 @@ def forcePackageCustomName(var):
 
 
 def forceProductType(var):
-	v = forceUnicodeLower(var)
-	if v in ('localboot', 'localbootproduct'):
+	lowercaseVar = forceUnicodeLower(var)
+	if lowercaseVar in ('localboot', 'localbootproduct'):
 		return u'LocalbootProduct'
-	elif v in ('netboot', 'netbootproduct'):
+	elif lowercaseVar in ('netboot', 'netbootproduct'):
 		return u'NetbootProduct'
 	else:
 		raise ValueError(u"Unknown product type: '%s'" % var)
@@ -471,16 +505,16 @@ def forceRequirementType(var):
 
 def forceObjectClass(var, objectClass):
 	exception = None
-	if type(var) in (types.UnicodeType, types.StringType) and var.lstrip() and var.lstrip().startswith('{'):
+	if isinstance(var, _STRING_TYPES) and var.lstrip() and var.lstrip().startswith('{'):
 		from OPSI.Util import fromJson
 
 		try:
 			var = fromJson(var)
-		except Exception as e:
-			exception = e
-			logger.debug(u"Failed to get object from json '%s': %s" % (var, e))
+		except Exception as error:
+			exception = error
+			logger.debug(u"Failed to get object from json '%s': %s" % (var, error))
 
-	if type(var) is types.DictType:
+	if isinstance(var, dict):
 		if 'type' not in var:
 			raise ValueError(u"Key 'type' missing in hash '%s'" % var)
 
@@ -489,12 +523,18 @@ def forceObjectClass(var, objectClass):
 			c = eval('OPSI.Object.%s' % var['type'])
 			if issubclass(c, objectClass):
 				var = c.fromHash(var)
-		except Exception as e:
-			exception = e
-			logger.debug(u"Failed to get object from dict '%s': %s" % (var, e))
+		except AttributeError as error:
+			if "'module' object has no attribute " in str(error):
+				error = ValueError("Invalild object type: {0}".format(var['type']))
+
+			exception = error
+			logger.debug(u"Failed to get object from dict '%s': %s" % (var, error))
+		except Exception as error:
+			exception = error
+			logger.debug(u"Failed to get object from dict '%s': %s" % (var, error))
 
 	if not isinstance(var, objectClass):
-		if exception:
+		if exception is not None:
 			raise ValueError(u"Not a %s: '%s': %s" % (objectClass, var, exception))
 		else:
 			raise ValueError(u"Not a %s: '%s'" % (objectClass, var))
@@ -715,7 +755,15 @@ class OpsiError(Exception):
 			return u"%s" % self.ExceptionShortDescription
 
 	def __repr__(self):
-		return unicode(self).encode("utf-8")
+		if self._message and self._message != u'None':
+			text = u"<{0}({1!r})>".format(self.__class__.__name__, self._message)
+		else:
+			text = u"<{0}()>".format(self.__class__.__name__)
+
+		if sys.version_info > (3, ):
+			return text
+		else:
+			return text.encode('utf-8')
 
 	__str__ = __repr__
 	complete_message = __unicode__
@@ -769,6 +817,32 @@ class OpsiTimeoutError(OpsiError):
 class OpsiProductOrderingError(OpsiError):
 	ExceptionShortDescription = u"A condition for ordering cannot be fulfilled"
 
+	def __init__(self, message='', problematicRequirements=None):
+		problematicRequirements = problematicRequirements or []
+
+		self._message = forceUnicode(message)
+		self.problematicRequirements = problematicRequirements
+
+	def __repr__(self):
+		if self._message and self._message != u'None':
+			text = u"<{0}({1!r}, {2!r})>".format(self.__class__.__name__, self._message, self.problematicRequirements)
+		else:
+			text = u"<{0}()>".format(self.__class__.__name__)
+
+		if sys.version_info > (3, ):
+			return text
+		else:
+			return text.encode('utf-8')
+
+	def __unicode__(self):
+		if self._message:
+			if self.problematicRequirements:
+				return u"{0}: {1} ({2})".format(self.ExceptionShortDescription, self._message, self.problematicRequirements)
+			else:
+				return u"{0}: {1}".format(self.ExceptionShortDescription, self._message)
+		else:
+			return forceUnicode(self.ExceptionShortDescription)
+
 
 class OpsiVersionError(OpsiError):
 	ExceptionShortDescription = u"Opsi version error"
