@@ -6,7 +6,7 @@ opsi python library - File.Opsi
 This module is part of the desktop management solution opsi
 (open pc server integration) http://www.opsi.org
 
-Copyright (C) 2006-2014 uib GmbH
+Copyright (C) 2006-2015 uib GmbH
 
 http://www.uib.de/
 
@@ -31,8 +31,6 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 :license: GNU General Public License version 2
 """
 
-__version__ = '4.0.6.1'
-
 import bz2
 import collections
 import datetime
@@ -44,6 +42,7 @@ import tempfile
 import shutil
 import socket
 import StringIO
+from contextlib import closing
 from hashlib import sha1
 from subprocess import Popen, PIPE, STDOUT
 
@@ -59,7 +58,7 @@ elif os.name == 'nt':
 
 import OPSI.System
 from OPSI.Logger import Logger
-from OPSI.Object import *
+from OPSI.Object import BoolProductProperty, LocalbootProduct, NetbootProduct, Product, ProductDependency, ProductProperty, UnicodeProductProperty
 from OPSI.Types import (BackendBadValueError, OpsiBackupBackendNotFound,
 	OpsiBackupFileError, OpsiBackupFileNotFound, forceActionRequest, forceBool,
 	forceDictList, forceFilename, forceHostId, forceInstallationStatus,
@@ -71,6 +70,8 @@ from OPSI.Types import (BackendBadValueError, OpsiBackupBackendNotFound,
 from OPSI.Util.File import ConfigFile, IniFile, TextFile, requiresParsing
 from OPSI.Util import md5sum, toJson, fromJson
 
+__version__ = '4.0.6.39'
+
 logger = Logger()
 
 
@@ -235,7 +236,7 @@ class BackendDispatchConfigFile(ConfigFile):
 		"""
 		Returns the dispatch config entries with RegEx and corresponding backends.
 
-		:returntype: [['regex',['backend1', 'backend2',]
+		:returntype: [['regex', ['backend1', 'backend2',]]]
 		"""
 		if lines:
 			self._lines = forceUnicodeList(lines)
@@ -249,10 +250,11 @@ class BackendDispatchConfigFile(ConfigFile):
 			if not match:
 				logger.error(u"Found bad formatted line '%s' in dispatch config file '%s'" % (line, self._filename))
 				continue
+
 			method = match.group(1).strip()
-			dispatch.append([method, []])
-			for entry in match.group(2).strip(',').split(','):
-				dispatch[-1][1].append(entry.strip())
+			backends = [entry.strip() for entry
+						in match.group(2).strip(',').split(',')]
+			dispatch.append([method, backends])
 		self._parsed = True
 
 		return dispatch
@@ -465,7 +467,7 @@ class PackageControlFile(TextFile):
 					value = forceBool(value)
 
 			elif (sectionType == 'product' and key in \
-					('id', 'type', 'name', 	'description', 'advice',
+					('id', 'type', 'name', 'description', 'advice',
 					 'version', 'packageversion', 'priority',
 					 'licenserequired', 'productclasses', 'pxeconfigtemplate',
 					 'setupscript', 'uninstallscript', 'updatescript',
@@ -533,8 +535,7 @@ class PackageControlFile(TextFile):
 				elif key == 'requirementtype':
 					value = forceRequirementType(value)
 
-			elif (sectionType == 'productproperty' and key in \
-					('type', 'name', 'default', 'values', 'description', 'editable', 'multivalue')):
+			elif sectionType == 'productproperty' and key in ('type', 'name', 'default', 'values', 'description', 'editable', 'multivalue'):
 				option = key
 				if key == 'type':
 					value = forceProductPropertyType(value)
@@ -560,7 +561,7 @@ class PackageControlFile(TextFile):
 			if not self._sections[sectionType][-1].has_key(option):
 				self._sections[sectionType][-1][option] = value
 			else:
-				if type(self._sections[sectionType][-1][option]) is unicode:
+				if isinstance(self._sections[sectionType][-1][option], unicode):
 					if not self._sections[sectionType][-1][option].endswith('\n'):
 						self._sections[sectionType][-1][option] += u'\n'
 					self._sections[sectionType][-1][option] += value.lstrip()
@@ -583,7 +584,7 @@ class PackageControlFile(TextFile):
 							# TODO: use set
 							tmp = []
 							for v in forceList(value):
-								if not v in tmp:
+								if v not in tmp:
 									tmp.append(v)
 							value = tmp
 						except Exception as error:
@@ -601,11 +602,11 @@ class PackageControlFile(TextFile):
 							# TODO: use set
 							tmp = []
 							for v in forceList(value):
-								if not v in ('', None) and v not in tmp:
+								if v not in ('', None) and v not in tmp:
 									tmp.append(v)
 							value = tmp
 
-					if type(value) is unicode:
+					if isinstance(value, unicode):
 						value = value.rstrip()
 
 					self._sections[sectionType][i][option] = value
@@ -620,7 +621,7 @@ class PackageControlFile(TextFile):
 					match = re.search('^\s*([^\(]+)\s*\(*\s*([^\)]*)\s*\)*', dep)
 					if not match.group(1):
 						raise Exception(u"Bad package dependency '%s' in control file" % dep)
-						continue
+
 					package = match.group(1).strip()
 					version = match.group(2)
 					condition = None
@@ -632,7 +633,7 @@ class PackageControlFile(TextFile):
 						condition = match.group(1)
 						if not condition:
 							condition = u'='
-						if not condition in (u'=', u'<', u'<=', u'>', u'>='):
+						if condition not in (u'=', u'<', u'<=', u'>', u'>='):
 							raise Exception(u"Bad condition string '%s' in package dependency" % condition)
 						version = match.group(2)
 					else:
@@ -667,7 +668,7 @@ class PackageControlFile(TextFile):
 			description=product.get('description'),
 			advice=product.get('advice'),
 			productClassIds=product.get('productclasses'),
-			windowsSoftwareIds=self._sections.get('windows',[{}])[0].get('softwareids', []),
+			windowsSoftwareIds=self._sections.get('windows', [{}])[0].get('softwareids', []),
 			changelog=self._sections.get('changelog')
 		)
 		if isinstance(self._product, NetbootProduct) and product.get('pxeconfigtemplate') is not None:
@@ -904,7 +905,6 @@ class PackageControlFile(TextFile):
 		if not self._opsi3compatible and self._product.getChangelog():
 			self._lines.append(u'[Changelog]')
 			self._lines.extend(self._product.getChangelog().split('\n'))
-			self._lines.append(u'')
 
 		self.open('w')
 		self.writelines()
@@ -947,9 +947,9 @@ class OpsiConfFile(IniFile):
 			if match:
 				sectionType = match.group(1).strip().lower()
 				if sectionType not in ('groups', 'packages'):
-					raise Exception(u"Parse error in line %s: unknown section '%s'" % (lineNum, sectionType))
+					raise ValueError(u"Parse error in line %s: unknown section '%s'" % (lineNum, sectionType))
 			elif not sectionType and line:
-				raise Exception(u"Parse error in line %s: not in a section" % lineNum)
+				raise ValueError(u"Parse error in line %s: not in a section" % lineNum)
 
 			key = None
 			value = None
@@ -1048,7 +1048,7 @@ class OpsiBackupArchive(tarfile.TarFile):
 
 		self._filemap = {}
 
-		assert(self.mode and str(self.mode)[0] in ("r", "w"))
+		assert self.mode and str(self.mode)[0] in ("r", "w")
 		tarfile.TarFile.__init__(self, name, self.mode, fileobj=fileobj, **kwargs)
 
 		if self.mode.startswith("w"):
@@ -1070,7 +1070,7 @@ class OpsiBackupArchive(tarfile.TarFile):
 			try:
 				dispatchedBackends = BackendDispatchConfigFile(self.DISPATCH_CONF).getUsedBackends()
 			except Exception as error:
-				logger.warning(u"Could not read dispatch configuration: %s" % unicode(error))
+				logger.warning(u"Could not read dispatch configuration: %s" % forceUnicode(error))
 				dispatchedBackends = []
 
 		if not os.path.exists(self.BACKEND_CONF_DIR):
@@ -1135,25 +1135,19 @@ class OpsiBackupArchive(tarfile.TarFile):
 
 	def _readSysInfo(self):
 		map = {}
-		fp = self.extractfile("%s/sysinfo" % self.CONTROL_DIR)
-		try:
+		with closing(self.extractfile("%s/sysinfo" % self.CONTROL_DIR)) as fp:
 			for line in fp.readlines():
 				key, value = line.split(":")
 				map[key.strip()] = value.strip()
-		finally:
-			fp.close()
 
 		return map
 
 	def _readChecksumFile(self):
 		map = {}
-		fp = self.extractfile("%s/checksums" % self.CONTROL_DIR)
-		try:
+		with closing(self.extractfile("%s/checksums" % self.CONTROL_DIR)) as fp:
 			for line in fp.readlines():
 				key, value = line.split(" ", 1)
 				map[value.strip()] = key.strip()
-		finally:
-			fp.close()
 
 		return map
 
@@ -1169,14 +1163,12 @@ class OpsiBackupArchive(tarfile.TarFile):
 		else:
 			checksum = sha1()
 
-			f = open(path)
-			chunk = True
-			try:
+			with open(path) as f:
+				chunk = True
 				while chunk:
 					chunk = f.read()
 					checksum.update(chunk)
-			finally:
-				f.close()
+
 			self._filemap[dest] = checksum.hexdigest()
 
 			self.add(path, dest)
@@ -1214,14 +1206,11 @@ class OpsiBackupArchive(tarfile.TarFile):
 
 				count = 0
 				chunk = True
-				fp = self.extractfile(member)
-				try:
+				with closing(self.extractfile(member)) as fp:
 					while chunk:
 						chunk = fp.read()
 						count += len(chunk)
 						filesum.update(chunk)
-				finally:
-					fp.close()
 
 				if checksum != filesum.hexdigest():
 					logger.debug2("Read %s bytes from file %s, resulting in checksum %s" % (count, member.name, filesum.hexdigest()))
@@ -1245,14 +1234,12 @@ class OpsiBackupArchive(tarfile.TarFile):
 			filesum = sha1()
 
 			chunk = True
-			fp = self.extractfile(member.name)
-			try:
+			with closing(self.extractfile(member.name)) as fp:
 				while chunk:
 					chunk = fp.read()
 					filesum.update(chunk)
 					os.write(tf, chunk)
-			finally:
-				fp.close()
+
 			if filesum.hexdigest() != checksum:
 				raise OpsiBackupFileError("Error restoring file %s: checksum missmacht.")
 
@@ -1442,7 +1429,7 @@ class OpsiBackupArchive(tarfile.TarFile):
 		for backend in self._getBackends("mysql"):
 			if not auto or backend["dispatch"]:
 				fd, name = tempfile.mkstemp(dir=self.tempdir)
-				os.chmod(name, 0770)
+				os.chmod(name, 0o770)
 
 				try:
 					for member in self.getmembers():
