@@ -4,7 +4,7 @@
 # This module is part of the desktop management solution opsi
 # (open pc server integration) http://www.opsi.org
 
-# Copyright (C) 2006-2015 uib GmbH <info@uib.de>
+# Copyright (C) 2006-2016 uib GmbH <info@uib.de>
 # http://www.uib.de/
 
 # This program is free software: you can redistribute it and/or modify
@@ -32,8 +32,6 @@ or to JSON, working with librsync and more.
 :license: GNU Affero General Public License version 3
 """
 
-__version__ = '4.0.6.1'
-
 import base64
 import codecs
 import json
@@ -45,8 +43,10 @@ import socket
 import struct
 import time
 import types
+from contextlib import closing
 from Crypto.Cipher import Blowfish
 from hashlib import md5
+from itertools import islice
 
 try:
 	import argparse
@@ -57,6 +57,8 @@ from OPSI.Logger import Logger
 from OPSI.Types import (forceBool, forceFilename, forceFqdn, forceInt,
 						forceIPAddress, forceNetworkAddress, forceUnicode)
 
+__version__ = '4.0.6.41'
+
 logger = Logger()
 
 if os.name == 'posix':
@@ -88,17 +90,17 @@ class PickleString(str):
 
 def deserialize(obj, preventObjectCreation=False):
 	newObj = None
-	if not preventObjectCreation and type(obj) is dict and 'type' in obj:
+	if not preventObjectCreation and isinstance(obj, dict) and 'type' in obj:
 		try:
 			import OPSI.Object
 			c = eval('OPSI.Object.%s' % obj['type'])
 			newObj = c.fromHash(obj)
-		except Exception as e:
-			logger.debug(u"Failed to get object from dict '%s': %s" % (obj, forceUnicode(e)))
+		except Exception as error:
+			logger.debug(u"Failed to get object from dict {0!r}: {1}".format(obj, forceUnicode(error)))
 			return obj
-	elif type(obj) is list:
+	elif isinstance(obj, list):
 		newObj = [deserialize(tempObject, preventObjectCreation=preventObjectCreation) for tempObject in obj]
-	elif type(obj) is dict:
+	elif isinstance(obj, dict):
 		newObj = {}
 		for (key, value) in obj.items():
 			newObj[key] = deserialize(value, preventObjectCreation=preventObjectCreation)
@@ -110,35 +112,38 @@ def deserialize(obj, preventObjectCreation=False):
 
 def serialize(obj):
 	newObj = None
-	if type(obj) in (unicode, str):
+	if isinstance(obj, (unicode, str)):
 		return obj
 	elif hasattr(obj, 'serialize'):
 		newObj = obj.serialize()
-	elif type(obj) is list:
+	elif isinstance(obj, (list, set, types.GeneratorType)):
 		newObj = [serialize(tempObject) for tempObject in obj]
-	elif type(obj) is dict:
+	elif isinstance(obj, dict):
 		newObj = {}
-		for (key, value) in obj.items():
+		for key, value in obj.items():
 			newObj[key] = serialize(value)
 	else:
 		return obj
+
 	return newObj
 
 
 def formatFileSize(sizeInBytes):
 	if sizeInBytes < 1024:
 		return '%i' % sizeInBytes
-	elif sizeInBytes < (1024**2):
+	elif sizeInBytes < 1048576:  # 1024**2
 		return '%iK' % (sizeInBytes / 1024)
-	elif sizeInBytes < (1024**3):
-		return '%iM' % (sizeInBytes / (1024**2))
+	elif sizeInBytes < 1073741824:  # 1024**3
+		return '%iM' % (sizeInBytes / 1048576)
+	elif sizeInBytes < 1099511627776:  # 1024**4
+		return '%iG' % (sizeInBytes / 1073741824)
 	else:
-		return '%iG' % (sizeInBytes / (1024**3))
+		return '%iT' % (sizeInBytes / 1099511627776)
 
 
 def fromJson(obj, objectType=None, preventObjectCreation=False):
 	obj = json.loads(obj)
-	if type(obj) is dict and objectType:
+	if isinstance(obj, dict) and objectType:
 		obj['type'] = objectType
 	return deserialize(obj, preventObjectCreation=preventObjectCreation)
 
@@ -148,20 +153,16 @@ def toJson(obj, ensureAscii=False):
 
 
 def librsyncSignature(filename, base64Encoded=True):
-	(f, sf) = (None, None)
 	try:
-		f = open(filename, 'rb')
-		sf = librsync.SigFile(f)
-		if base64Encoded:
-			sig = base64.encodestring(sf.read())
-		else:
-			sig = sf.read()
-		f.close()
-		sf.close()
-		return sig
+		with open(filename, 'rb') as f:
+			with closing(librsync.SigFile(f)) as sf:
+				sig = sf.read()
+
+				if base64Encoded:
+					sig = base64.encodestring(sig)
+
+				return sig
 	except Exception as e:
-		if f: f.close()
-		if sf: sf.close()
 		raise Exception(u"Failed to get librsync signature: %s" % forceUnicode(e))
 
 
@@ -174,48 +175,31 @@ def librsyncPatchFile(oldfile, deltafile, newfile):
 	if deltafile == oldfile:
 		raise ValueError(u"oldfile and deltafile are the same file")
 
-	(of, df, nf, pf) = (None, None, None, None)
-	bufsize = 1024*1024
+	bufsize = 1024 * 1024
 	try:
-		of = open(oldfile, "rb")
-		df = open(deltafile, "rb")
-		nf = open(newfile, "wb")
-		pf = librsync.PatchedFile(of, df)
-		data = True
-		while(data):
-			data = pf.read(bufsize)
-			nf.write(data)
-		nf.close()
-		pf.close()
-		df.close()
-		of.close()
+		with open(oldfile, "rb") as of:
+			with open(deltafile, "rb") as df:
+				with open(newfile, "wb") as nf:
+					with closing(librsync.PatchedFile(of, df)) as pf:
+						data = True
+						while data:
+							data = pf.read(bufsize)
+							nf.write(data)
 	except Exception as e:
-		if nf: nf.close()
-		if pf: pf.close()
-		if df: df.close()
-		if of: of.close()
 		raise Exception(u"Failed to patch file: %s" % forceUnicode(e))
 
 
 def librsyncDeltaFile(filename, signature, deltafile):
-	(f, df, ldf) = (None, None, None)
-	bufsize = 1024*1024
+	bufsize = 1024 * 1024
 	try:
-		f = open(filename, "rb")
-		df = open(deltafile, "wb")
-		ldf = librsync.DeltaFile(signature, f)
-
-		data = True
-		while(data):
-			data = ldf.read(bufsize)
-			df.write(data)
-		df.close()
-		f.close()
-		ldf.close()
+		with open(filename, "rb") as f:
+			with open(deltafile, "wb") as df:
+				with closing(librsync.DeltaFile(signature, f)) as ldf:
+					data = True
+					while data:
+						data = ldf.read(bufsize)
+						df.write(data)
 	except Exception as e:
-		if df:  df.close()
-		if f:   f.close()
-		if ldf: ldf.close()
 		raise Exception(u"Failed to write delta file: %s" % forceUnicode(e))
 
 
@@ -255,9 +239,8 @@ def generateOpsiHostKey(forcePython=False):
 	key = u''
 	if os.name == 'posix' and not forcePython:
 		logger.debug(u"Opening random device '%s' to generate opsi host key" % RANDOM_DEVICE)
-		r = open(RANDOM_DEVICE)
-		key = r.read(16)
-		r.close()
+		with open(RANDOM_DEVICE) as r:
+			key = r.read(16)
 		logger.debug("Random device closed")
 		key = unicode(key.encode("hex"))
 	else:
@@ -284,13 +267,13 @@ def objectToBeautifiedText(obj, level=0):
 	text = []
 	append = text.append
 
-	if type(obj) is types.ListType:
+	if isinstance(obj, (list, set)):
 		append(indent)
 		append(u'[\n')
 
 		if obj:
 			for element in obj:
-				if not type(element) in (types.DictType, types.ListType):
+				if not isinstance(element, (dict, list)):
 					append(indent)
 				append(objectToBeautifiedText(element, level + 1))
 				append(u',')
@@ -300,7 +283,7 @@ def objectToBeautifiedText(obj, level=0):
 
 		append(indent)
 		append(u']')
-	elif type(obj) is types.DictType:
+	elif isinstance(obj, dict):
 		append(indent)
 		append(u'{\n')
 
@@ -308,7 +291,7 @@ def objectToBeautifiedText(obj, level=0):
 			for (key, value) in obj.iteritems():
 				append(indent)
 				append(key.join((u'"', u'" : ')))
-				if type(value) in (types.DictType, types.ListType):
+				if isinstance(value, (dict, list)):
 					append(u'\n')
 				append(objectToBeautifiedText(value, level + 1))
 				append(u',')
@@ -318,7 +301,7 @@ def objectToBeautifiedText(obj, level=0):
 
 		append(indent)
 		append(u'}')
-	elif type(obj) is str:
+	elif isinstance(obj, str):  # TODO: watch out for Python 3
 		append(toJson(forceUnicode(obj)))
 	else:
 		append(toJson(obj))
@@ -346,10 +329,10 @@ def objectToBash(obj, bashVars=None, level=0):
 	if hasattr(obj, 'serialize'):
 		obj = obj.serialize()
 
-	if type(obj) is types.ListType:
+	if isinstance(obj, (list, set)):
 		bashVars[varName] += u'(\n'
 		for i in range( len(obj) ):
-			if type(obj[i]) in (types.DictType, types.ListType):
+			if isinstance(obj[i], (dict, list)):
 				hashFound = True
 				level += 1
 				objectToBash(obj[i], bashVars, level)
@@ -358,11 +341,11 @@ def objectToBash(obj, bashVars=None, level=0):
 				objectToBash(obj[i], bashVars, level)
 			bashVars[varName] += u'\n'
 		bashVars[varName] = bashVars[varName][:-1] + u'\n)'
-	elif type(obj) is types.DictType:
+	elif isinstance(obj, dict):
 		bashVars[varName] += u'(\n'
 		for (key, value) in obj.items():
 			bashVars[varName] += '%s=' % key
-			if type(value) in (types.DictType, types.ListType):
+			if isinstance(value, (dict, list)):
 				level += 1
 				v = objectToBash(value, bashVars, level)
 				bashVars[varName] += u'${RESULT%d[*]}' % level
@@ -387,37 +370,35 @@ def objectToHtml(obj, level=0):
 	html = []
 	append = html.append
 
-	if type(obj) is types.ListType:
+	if isinstance(obj, (list, set)):
 		append(u'[')
 		if len(obj) > 0:
 			append(u'<div style="padding-left: 3em;">')
-			for i in range(len(obj)):
-				append(objectToHtml(obj[i], level+1))
+			for i, currentElement in enumerate(obj):
+				append(objectToHtml(currentElement, level + 1))
 				if i < len(obj) - 1:
 					append(u',<br />\n')
 			append(u'</div>')
 		append(u']')
-	elif type(obj) is types.DictType:
+	elif isinstance(obj, dict):
 		append(u'{')
 		if len(obj) > 0:
 			append(u'<div style="padding-left: 3em;">')
-			i = 0
-			for (key, value) in obj.items():
+			for i, (key, value) in enumerate(obj.items()):
 				append(u'<font class="json_key">')
 				append(objectToHtml(key))
 				append(u'</font>: ')
-				append(objectToHtml(value, level+1))
+				append(objectToHtml(value, level + 1))
 				if i < len(obj) - 1:
 					append(u',<br />\n')
-				i += 1
 			append(u'</div>')
 		append(u'}')
-	elif type(obj) is types.BooleanType:
+	elif isinstance(obj, bool):
 		append(str(obj).lower())
-	elif type(obj) is types.NoneType:
+	elif obj is None:
 		append('null')
 	else:
-		if type(obj) in (str, unicode):
+		if isinstance(obj, (str, unicode)):  # TODO: watch out for Python 3
 			append(replaceSpecialHTMLCharacters(obj).join((u'"', u'"')))
 		else:
 			append(replaceSpecialHTMLCharacters(obj))
@@ -429,12 +410,12 @@ def replaceSpecialHTMLCharacters(text):
 	return forceUnicode(text)\
 		.replace(u'\r', u'')\
 		.replace(u'\t', u'   ')\
-		.replace(u'&',  u'&amp;')\
-		.replace(u'"',  u'&quot;')\
-		.replace(u"'",  u'&apos;')\
-		.replace(u' ',  u'&nbsp;')\
-		.replace(u'<',  u'&lt;')\
-		.replace(u'>',  u'&gt;')\
+		.replace(u'&', u'&amp;')\
+		.replace(u'"', u'&quot;')\
+		.replace(u"'", u'&apos;')\
+		.replace(u' ', u'&nbsp;')\
+		.replace(u'<', u'&lt;')\
+		.replace(u'>', u'&gt;')\
 		.replace(u'\n', u'<br />\n')
 
 
@@ -513,9 +494,9 @@ def compareVersions(v1, condition, v2):
 					logger.debug2(u"%s == %s => continue" % (cv1, cv2))
 					continue
 
-				if type(cv1) is not int:
+				if not isinstance(cv1, int):
 					cv1 = u"'%s'" % cv1
-				if type(cv2) is not int:
+				if not isinstance(cv2, int):
 					cv2 = u"'%s'" % cv2
 
 				b = eval(u"%s %s %s" % (cv1, condition, cv2))
@@ -624,8 +605,8 @@ def blowfishDecrypt(key, crypt):
 
 def encryptWithPublicKeyFromX509CertificatePEMFile(data, filename):
 	import M2Crypto
-	f = open(filename, 'r')
-	try:
+
+	with open(filename, 'r') as f:
 		cert = M2Crypto.X509.load_cert_string(f.read())
 		rsa = cert.get_pubkey().get_rsa()
 		enc = ''
@@ -637,8 +618,6 @@ def encryptWithPublicKeyFromX509CertificatePEMFile(data, filename):
 		for chunk in chunks:
 			enc += rsa.public_encrypt(data=chunk, padding=M2Crypto.RSA.pkcs1_oaep_padding)
 		return enc
-	finally:
-		f.close()
 
 
 def decryptWithPrivateKeyFromPEMFile(data, filename):
@@ -652,7 +631,8 @@ def decryptWithPrivateKeyFromPEMFile(data, filename):
 	res = ''
 	for chunk in chunks:
 		res += privateKey.private_decrypt(data=chunk, padding=M2Crypto.RSA.pkcs1_oaep_padding)
-	if res.find('\0') != -1:
+
+	if '\0' in res:
 		res = res[:res.find('\0')]
 	return res
 
@@ -699,7 +679,7 @@ def findFiles(directory, prefix=u'', excludeDir=None, excludeFile=None, includeD
 
 	files = []
 	for entry in listdir(directory):
-		if type(entry) is str:
+		if isinstance(entry, str):  # TODO how to handle this with Python 3?
 			logger.error(u"Bad filename '%s' found in directory '%s', skipping entry!" % (unicode(entry, 'ascii', 'replace'), directory))
 			continue
 		pp = os.path.join(prefix, entry)
@@ -721,16 +701,18 @@ def findFiles(directory, prefix=u'', excludeDir=None, excludeFile=None, includeD
 				files.append(pp)
 			files.extend(
 				findFiles(
-					directory   = dp,
-					prefix      = pp,
-					excludeDir  = excludeDir,
-					excludeFile = excludeFile,
-					includeDir  = includeDir,
-					includeFile = includeFile,
-					returnDirs  = returnDirs,
-					returnLinks = returnLinks,
-					followLinks = followLinks,
-					repository  = repository) )
+					directory=dp,
+					prefix=pp,
+					excludeDir=excludeDir,
+					excludeFile=excludeFile,
+					includeDir=includeDir,
+					includeFile=includeFile,
+					returnDirs=returnDirs,
+					returnLinks=returnLinks,
+					followLinks=followLinks,
+					repository=repository
+				)
+			)
 			continue
 
 		if excludeFile and re.search(excludeFile, entry):
@@ -756,43 +738,62 @@ def isRegularExpressionPattern(object):
 
 
 def ipAddressInNetwork(ipAddress, networkAddress):
+	"""
+	Checks if the given IP address is in the given network range.
+	Returns ``True`` if the given address is part of the network.
+	Returns ``False`` if the given address is not part of the network.
+
+	:param ipAddress: The IP which we check.
+	:type ipAddress: str
+	:param networkAddress: The network address written with slash notation.
+	:type networkAddress: str
+	"""
+	def createBytemaskFromAddress(address):
+		"Returns an int representation of an bytemask of an ipAddress."
+		num = [forceInt(part) for part in address.split('.')]
+		return (num[0] << 24) + (num[1] << 16) + (num[2] << 8) + num[3]
+
 	ipAddress = forceIPAddress(ipAddress)
 	networkAddress = forceNetworkAddress(networkAddress)
 
-	n = ipAddress.split('.')
-	for i in range(4):
-		n[i] = forceInt(n[i])
-	ip = (n[0] << 24) + (n[1] << 16) + (n[2] << 8) + n[3]
+	ip = createBytemaskFromAddress(ipAddress)
 
-	(network, netmask) = networkAddress.split(u'/')
-	while (network.count('.') < 3):
-		network = network + u'.0'
-	if (netmask.find('.') == -1):
-		netmask = forceUnicode(socket.inet_ntoa(struct.pack('>I',0xffffffff ^ (1 << 32 - forceInt(netmask)) - 1)))
-	while (netmask.count('.') < 3):
+	network, netmask = networkAddress.split(u'/')
+
+	if '.' not in netmask:
+		netmask = forceUnicode(socket.inet_ntoa(struct.pack('>I', 0xffffffff ^ (1 << 32 - forceInt(netmask)) - 1)))
+
+	while netmask.count('.') < 3:
 		netmask = netmask + u'.0'
 
-	logger.debug(u"Testing if ip %s is part of network %s/%s" % (ipAddress, network, netmask))
+	logger.debug(
+		u"Testing if ip {ipAddress} is part of network "
+		u"{network}/{netmask}".format(
+			ipAddress=ipAddress,
+			network=network,
+			netmask=netmask
+		)
+	)
 
-	n = network.split('.')
-	for i in range(4):
-		n[i] = int(n[i])
-	network = (n[0] << 24) + (n[1] << 16) + (n[2] << 8) + n[3]
-	n = netmask.split('.')
-	for i in range(4):
-		n[i] = int(n[i])
-	netmask = (n[0] << 24) + (n[1] << 16) + (n[2] << 8) + n[3]
+	network = createBytemaskFromAddress(network)
+	netmask = createBytemaskFromAddress(netmask)
 
-	wildcard = netmask ^ 0xFFFFFFFFL
-	if (wildcard | ip == wildcard | network):
+	wildcard = netmask ^ 0xFFFFFFFF
+	if wildcard | ip == wildcard | network:
 		return True
+
 	return False
 
 
 def flattenSequence(sequence):
+	"""
+	Flattens nested sequences so that only a flat list will be returned.
+
+	:returntype: list
+	"""
 	listToReturn = []
 	for part in sequence:
-		if type(part) in (types.ListType, types.TupleType):
+		if isinstance(part, (list, tuple, set, types.GeneratorType)):
 			listToReturn.extend(flattenSequence(part))
 		else:
 			listToReturn.append(part)
@@ -868,3 +869,17 @@ def removeDirectory(directory):
 
 		import OPSI.System  # late import to avoid circular dependency
 		OPSI.System.execute('rm -rf {dir}'.format(dir=directory))
+
+
+def chunk(iterable, size):
+	"""
+	Returns chunks (parts) of a specified `size` from `iterable`.
+	It will not pad (fill) the chunks.
+
+	This works lazy and therefore can be used with any iterable without
+	much overhead.
+
+	Original recipe from http://stackoverflow.com/a/22045226
+	"""
+	it = iter(iterable)
+	return iter(lambda: tuple(islice(it, size)), ())
