@@ -1310,10 +1310,6 @@ class BackendContentFixture(Fixture):
 			self.backend.softwareLicenseToLicensePool_createObjects(self.softwareLicenseToLicensePools)
 			self.backend.licenseOnClient_createObjects(self.licenseOnClients)
 
-class FileBackendConfigFixtire(Fixture):
-
-	template = None
-
 
 class FileBackendFixture(_BackendFixture):
 
@@ -1359,57 +1355,6 @@ class FileBackendFixture(_BackendFixture):
 		self.options.update(options)
 		self.backend.backend_setOptions(self.options)
 
-class SQLiteBackendFixture(_BackendFixture):
-
-	defaultOptions = {
-			'processProductPriorities':            True,
-			'processProductDependencies':          True,
-			'addProductOnClientDefaults':          True,
-			'addProductPropertyStateDefaults':     True,
-			'addConfigStateDefaults':              True,
-			'deleteConfigStateIfDefault':          True,
-			'returnObjectsOnUpdateAndCreate':      False
-	}
-	licenseManagement = True
-
-	def __init__(self, database=":memory:"):
-		super(SQLiteBackendFixture, self).__init__()
-
-		self.database = database
-		self.backend = None
-
-	def setupBackend(self):
-
-		hw = HwAuditConfigFixture()
-
-		self.useFixture(hw)
-
-		if not self.database:
-			bd = self.useFixture(TempDir())
-			self.baseDir = bd.path
-			self.database = os.path.join(self.baseDir, "opsi.sqlite")
-
-		self.backend = SQLiteBackend(database=self.database, audithardwareconfigfile=hw.path)
-		self.extend()
-
-class SQLiteModificationTrackerFixture(Fixture):
-
-	def __init__(self, database=":memory:"):
-		super(SQLiteModificationTrackerFixture, self).setUp()
-
-		self.database = database
-		self.tracker = None
-
-	def setUp(self):
-		super(SQLiteModificationTrackerFixture, self).setUp()
-		if not self.database:
-			bd = self.useFixture(TempDir())
-			self.baseDir = bd.path
-			self.database = os.path.join(self.baseDir, "tracker.sqlite")
-
-		self.tracker = SQLiteObjectBackendModificationTracker(database = self.database)
-
-
 
 class MySQLBackendFixture(_BackendFixture):
 
@@ -1469,38 +1414,6 @@ class MySQLBackendFixture(_BackendFixture):
 		del self.mb._sql._pool
 
 
-class LDAPBackendFixture(_BackendFixture):
-
-	baseDn = u'dc=uib,dc=local'
-
-	def __init__(self, username="admin", password="linux123", address="stb-40-srv-050", opsiBaseDn="opsi-test", hostsContainerDN="hosts"):
-
-		super(LDAPBackendFixture, self).__init__()
-
-		self.username = u"cn=%s,%s" % (username, self.baseDn)
-		self.password = password
-		self.address = address
-
-		self.opsiBaseDn = u"cn=%s,%s" % (opsiBaseDn,self.baseDn)
-		self.hostsContainerDn = u"cn=%s,%s" % (hostsContainerDN, self.opsiBaseDn)
-
-	def setupBackend(self):
-
-		hw = HwAuditConfigFixture()
-		self.useFixture(hw)
-
-		self.backend = LDAPBackend(
-					username         = self.username,
-					password         = self.password,
-					address          = self.address,
-					opsiBaseDn       = self.opsiBaseDn,
-					hostsContainerDn = self.hostsContainerDn,
-					audithardwareconfigfile=hw.path
-					)
-		self.addCleanup(self.backend.backend_deleteBase)
-
-		self.extend()
-
 class BackendTestCase(TestCase):
 
 	inventoryHistory = False
